require 'sketchup.rb'
module DM
module ProfileBuilder2
class LatheTool
include Geometry
GETFIRSTPOINT=0
GETSECONDPOINT=1
@@dict="ProfileBuilder"
def initialize(profile)
		
	@profile=profile.dup
    @ip1 = nil
    @ip2 = nil
    @xdown = 0
    @ydown = 0
	@angle=Math::PI*2.0
	@num_segments=get_last_num_segments()
	@profile_revolver=ProfileRevolver.new(@profile)
	@profile_revolver.material=nil
	plugins=PATH
	icon=File.join(plugins,"cursors/revolve_cursor.png")
	@cursor=UI.create_cursor(icon,1,1)
end
def get_last_num_segments()
num=Sketchup.active_model.get_attribute(@@dict,'revolve_segments',24)
return num
end
def activate
    @ip1 = Sketchup::InputPoint.new
    @ip2 = Sketchup::InputPoint.new
    @ip = Sketchup::InputPoint.new
    @drawn = false
    self.reset(nil)
end
def deactivate(view)
    view.invalidate if @drawn
end
#######################
def onSetCursor
UI.set_cursor(@cursor) if @cursor
end

def getExtents
box=Sketchup.active_model.bounds
box.add(@ip1.position) if @ip1.valid?
box.add(@ip.position) if @ip.valid?
box.add(@ip2.position) if @ip2.valid?
return box
end
def resume(view)
set_status_text()
end
def onMouseMove(flags, x, y, view)
    if( @state == GETFIRSTPOINT )
   
        @ip.pick view, x, y
        if( @ip != @ip1 )
           
            view.invalidate if( @ip.display? or @ip1.display? )
            @ip1.copy! @ip
            view.tooltip = @ip1.tooltip
        end
    else
  
        @ip2.pick view, x, y, @ip1
        view.tooltip = @ip2.tooltip if( @ip2.valid? )
        view.invalidate
        
        if( (x-@xdown).abs > 10 || (y-@ydown).abs > 10 )
            @dragging = true
        end
    end
end
def onLButtonDown(flags, x, y, view)
    
    if( @state == GETFIRSTPOINT )
        @ip1.pick view, x, y
        if( @ip1.valid? )
            @state = GETSECONDPOINT
            Sketchup::set_status_text("Select second end", SB_PROMPT)
            @xdown = x
            @ydown = y
        end
    else
       
        if( @ip2.valid? )
            self.revolve_profile(@ip1.position, @ip2.position,view)
            self.reset(view)
        end
    end
    
    view.lock_inference
end
def onLButtonUp(flags, x, y, view)
    if( @dragging && @ip2.valid? )
        self.revolve_profile(@ip1.position, @ip2.position,view)
        self.reset(view)
    end
end
def onKeyDown(key, repeat, flags, view)
    if( key == CONSTRAIN_MODIFIER_KEY && repeat == 1 )
        @shift_down_time = Time.now
        
        if( view.inference_locked? )
            view.lock_inference
        elsif( @state == GETFIRSTPOINT && @ip1.valid? )
            view.lock_inference @ip1
        elsif( @state == GETSECONDPOINT && @ip2.valid? )
            view.lock_inference @ip2, @ip1
        end
    end
end
def onKeyUp(key, repeat, flags, view)
    if( key == CONSTRAIN_MODIFIER_KEY &&
        view.inference_locked? &&
        (Time.now - @shift_down_time) > 0.5 )
        view.lock_inference
    end
end
def onUserText(text, view)
    #return if not @state == GETSECONDPOINT
    #return if not @ip2.valid?
    
    begin
        value = text.to_i
    rescue
        # Error parsing the text
    end
	if !value or value<3
		UI.messagebox("Minimum number of divisions is 3")
		return
	end
	@num_segments=value
	view.invalidate
end
def draw(view)
    if( @ip1.valid? )
        if( @ip1.display? )
            @ip1.draw(view)
            @drawn = true
        end
        
        if( @ip2.valid? )
            @ip2.draw(view) if( @ip2.display? )
            
            view.set_color_from_line(@ip1, @ip2)
            self.draw_geometry(@ip1.position, @ip2.position, view)
            @drawn = true
        end
    end
end
def onCancel(flag, view)
    self.reset(view)
end
def reset(view)
     @state = GETFIRSTPOINT
    
    set_status_text()
	
    # clear the InputPoints
    @ip1.clear
    @ip2.clear
    
    if( view )
        view.tooltip = nil
        view.invalidate if @drawn
    end
    
    @drawn = false
    @dragging = false
end
def set_status_text()
case @state
	when GETFIRSTPOINT
		Sketchup::set_status_text("点击轴的第一个点")
		Sketchup::set_status_text("Sides",SB_VCB_LABEL)
	when GETSECONDPOINT
		Sketchup::set_status_text("点击轴的第二个点")
		Sketchup::set_status_text("Sides",SB_VCB_LABEL)
end
Sketchup::set_status_text(@num_segments,SB_VCB_VALUE)
end
# Create new geometry when the user has selected two points.
def revolve_profile(p1, p2, view)
    model=view.model
	model.start_operation("Revolve Profile",true) 
	entities=model.active_entities
	origin=p1
	axis=p1.vector_to(p2)
	trans=Geom::Transformation.new(origin,axis)
	chain=[]
	pm=ProfileMember.add(entities,chain,trans)
	pm.name=@profile.name
	pm.set_from_profile!(@profile)
	pm.revolve_segments=@num_segments
	pm.revolve()
	model.set_attribute(@@dict,'revolve_segments',@num_segments)
	model.commit_operation
	
end
# Draw the geometry
def draw_geometry(pt1, pt2, view)
    view.draw_line(pt1, pt2)
	case @state
	when GETFIRSTPOINT
		#
	when GETSECONDPOINT
		preview_revolve(view)
	end
end
##
def preview_revolve(view)
origin=@ip1.position
axis=origin.vector_to(@ip2.position)
return if axis.length==0.0
trans=Geom::Transformation.new(origin,axis)
mesh,uv_hash=@profile_revolver.get_revolve_mesh(@num_segments)
mesh.transform!(trans)
polys=mesh.polygons
polys.each {|poly|
	if poly.length==3
		view.draw(GL_LINE_LOOP,mesh.point_at(poly[0]),mesh.point_at(poly[1]),mesh.point_at(poly[2]))
	else
		view.draw(GL_LINE_LOOP,mesh.point_at(poly[0]),mesh.point_at(poly[1]),mesh.point_at(poly[2]),mesh.point_at(poly[3]))
	end
}
end
end # class LatheTool

###
class ProfileRevolver
include Geometry
def initialize(profile)
@profile=profile
@angle=Math::PI*2.0
end
##
def material=(mat)
@material=mat
if @material
	if @material.texture
		@tex_width=@material.texture.width
		@tex_height=@material.texture.height
		@has_texture=true
		return
	end
end
@tex_width=1.0
@tex_height=1.0
end
##
def get_revolve_mesh(num_segments=24)
profile_pts=@profile.points
num_pts=profile_pts.length
is_1d=@profile.is_1d?
start_direction=Y_AXIS
origin=Geom::Point3d.new(0,0,0)
mesh=Geom::PolygonMesh.new
uv_hash={}
profile_pts=@profile.get_transformed_loop(profile_pts)
trans=transform_by_placement_point(profile_pts,7)
profile_pts=profile_pts.collect {|p| p.transform(trans)}
trans=Geom::Transformation.new(origin,Y_AXIS)
profile_pts=profile_pts.collect {|p| p.transform(trans)}
bb=Geom::BoundingBox.new
bb.add(profile_pts)
radius=bb.width
segments=num_segments/(2.0*Math::PI)*@angle
max_segment_distance=Math::PI*2.0*radius/segments
u_step=max_segment_distance/@tex_width
num_wraps=(Math::PI*2.0*radius/@tex_width).round.to_f
u_step=num_wraps/segments
segments=segments.to_i
angle_step=@angle/(segments.to_f)
start_pts=profile_pts.dup
trans_step=Geom::Transformation.rotation(Geom::Point3d.new(0,0,0),Z_AXIS,angle_step)
current_v_val=0.0
current_u_val=0.0
segments.times {|i|
	current_v_val=0.0
	next_pts=start_pts.collect {|p| p.transform(trans_step)}
	
	#stitch the edges
	num_pts.times {|j|
		if is_1d
			break if j==(num_pts-1)
		end
		
		j=j-num_pts if j==(num_pts-1)
		
		p0=start_pts[j]
		p1=next_pts[j]
		p2=next_pts[j+1]
		p3=start_pts[j+1]
		
		if p0[0]==0.0 and p0[1]==0.0 and p3[0]==0.0 and p3[1]==0.0
			next
		end
		
		i0=mesh.add_point(p0)
		i1=mesh.add_point(p1)
		i2=mesh.add_point(p2)
		i3=mesh.add_point(p3)
		
		l1=[p0,p1]
		l2=[p3,p2]
		pts=Geom.closest_points(l1,l2)
		v_offset=(p0.distance(p3))/@tex_height
		next_v_val=current_v_val+v_offset
		u1=current_u_val+u_step
		
		v03=p0.vector_to(p3)
		v01=p0.vector_to(p1)
		v01.normalize!
	
		u3=current_u_val
		u2=current_u_val+u_step
		
		uvs={}
		uvs[i0]=[current_u_val,current_v_val,1]
		uvs[i1]=[u1,current_v_val,1]
		uvs[i2]=[u2,next_v_val,1]
		uvs[i3]=[u3,next_v_val,1]
		
		new_poly=[i0,i1,i2,i3]
		duplicate_indices=[]
		new_poly.each_index {|i|
			new_poly.each_index {|j|
				if (new_poly[i]==new_poly[j]) and (i!=j)
					duplicate_indices=[i,j]
					break
				end
			}
		if duplicate_indices.length>0
			uvs[new_poly[duplicate_indices[0]]].x=(current_u_val+(u_step/2.0))
			uvs[new_poly[duplicate_indices[1]]].x=(current_u_val+(u_step/2.0))
			break
		end
		}
		
		new_poly.uniq!
		
		if new_poly.length>=3
			poly_index=mesh.add_polygon(new_poly) 
			uv_hash[poly_index]=uvs
		end
		
		current_v_val=next_v_val
	}
	current_u_val=current_u_val+u_step
	start_pts=next_pts.dup
}
return mesh,uv_hash

end

end #class revolver
end #module ProfileBuilder2

end #module DM
