require 'sketchup.rb'
module DM
module ProfileBuilder2
module Geometry
##GEOM
def point_on_plane?(plane,p)
value=plane[0]*p.x+plane[1]*p.y+plane[2]*p.z+plane[3]
return value
end

##
def get_inst_below_cursor(view,x,y,active=true)
pickray=view.pickray(x,y)
model=view.model
rt=model.raytest(pickray)
active_ents=Sketchup.active_model.active_entities
if rt
	path=rt[1]
	ent=path[-2]
	if (ent.class==Sketchup::Group or ent.class==Sketchup::ComponentInstance)
		if ((active) and (active_ents.include?(ent)))
			return ent
		elsif (!active)
			return ent
		end
	end
end
return nil
end
##
##
def get_trans_below_cursor(view,x,y,active=true)
pickray=view.pickray(x,y)
model=view.model
rt=model.raytest(pickray)
trans=Geom::Transformation.new
if rt
	path=rt[1]
	ent=path.pop()
	path.each {|inst| trans=trans*inst.transformation}	
end
return trans
end
##
def draw_picked_face(view,face,trans, polyline=false)
if face
	if face.class==Sketchup::Face
		return if !face.valid?
		verts=face.outer_loop.vertices
	elsif face.class==Array
		verts=face
	else
		raise ArgumentError,"Face or Array expected",caller
	end
	view.drawing_color=@highlight_color
	view.line_width=24
	view.line_stipple=""
	pts=verts.collect {|v| v.position.transform(trans) if v.valid?}
	#p polyline
	view.draw(GL_LINE_LOOP,pts) if !polyline
	view.draw(GL_LINE_STRIP,pts) if polyline
end

end
###
def point_on_edge(pt,edge)
dist=nil
p0=edge[0]
p1=edge[1]
dist=0.0 if pt==p0
edge_vec=p0.vector_to(p1)
edge_vec_length=edge_vec.length
dist=edge_vec_length if pt==p1
if !dist
	start_to_pt=p0.vector_to(pt).length
	pt_to_end=pt.vector_to(p1).length
	if (start_to_pt+pt_to_end)==edge_vec_length
		dist=start_to_pt
	else
		dist=nil
	end
end
return dist
end
def get_max_perp_edge_length(verts)
range=get_range_perpindicular_edge_length(verts)
return range[1]
end
##
def get_min_perp_edge_length(verts)
range=get_range_perpindicular_edge_length(verts)
return range[0]
end
###
def get_range_perpindicular_edge_length(verts)
perp_edges=get_perpindicular_edges(verts)
perp_edges_lengths=perp_edges.collect {|pe| pe.length.to_f}
return [perp_edges_lengths.min,perp_edges_lengths.max]
end
##
def get_perpindicular_edges(verts)
perp_edges=[]
adj_edges=verts.collect {|v| v.edges}
adj_edges.flatten!
adj_edges.delete_if {|edge| verts.include?(edge.start)&&verts.include?(edge.end)}
return adj_edges
end
##
def get_perpindicular_edges_old(verts)
perp_edges=[]
normal=get_normal_of_vertices(verts)
verts.each {|v|
	edges=v.edges
	perp_edge=edges.find {|edge| edge.line[1].parallel?(normal)}
	perp_edges.push(perp_edge)
}
return perp_edges
end
##
def get_max_distance_to_plane(verts,plane,trans)
vectors=[]
normal=get_normal_of_vertices(verts).transform(trans)
verts.each {|v|
	p1=v.position.transform(trans)
	line=[p1,normal]
	p2=Geom.intersect_line_plane(line,plane)
	if p2==nil
		return nil  #does not intersect
	end
	vectors.push(p1.vector_to(p2))
}
vector_lengths=vectors.collect {|vec| vec.length}
max_length=vector_lengths.max
if max_length==0.0
	return max_length
end
vec_index=vector_lengths.index(max_length)
max_vector=vectors[vec_index]
if !max_vector.samedirection?(normal)
	max_length=-(max_length)
end
return max_length
end

###nudge the origin of the ray to avoid false intersections
def nudge_ray(ray)
nudge_dist=0.01
vec=ray[1]
pt=ray[0]
vec.length=nudge_dist
pt=pt.offset(vec)
return [pt,ray[1]]
end
##
def divide_edge(edge)
p0=edge[0]
p1=edge[1]
new_pts=[]
vec=p0.vector_to(p1)
v_len=vec.length
p=Geom::Point3d.new(p0)
new_pts.push(p)
divs=(v_len/0.1).floor
if v_len>0.0
	vec.length=(v_len)/(divs.to_f)
	new_pts=[]
	(divs-1).times {|i|
		p=Geom::Point3d.new(p.offset(vec))
		new_pts.push(p)
	}
else
	new_pts=edge
end
new_pts.push(Geom::Point3d.new(p1))
return new_pts
end
##
def get_max_perp_distance_from_face_to_entity(face,entity)
if face
	model=Sketchup.active_model
	trans=self.transformation
	face_normal=face.normal.transform(trans)
	face_verts=face.outer_loop.vertices
	face_pts=face_verts.collect {|v| v.position.transform(trans)}
	chain=Chain.new(face_pts)
	chain.add(face_pts[0])  #close the loop of the chain
	edges=chain.edges
	face_pts=edges.collect {|edge| divide_edge(edge)}
	face_pts.flatten!
	
	face_rays=face_pts.collect {|p| [p,face_normal]}
	face_rays=face_rays.collect {|ray| nudge_ray(ray)}
	hit_entity=[]
	distance_to_entity=[]
	face_rays.each_index {|index|
		item=model.raytest(face_rays[index])
		running_distance=0.0
		current_ray_origin=face_pts[index]
		
		while item 
			path_array=item[1]
			hit_point=item[0]
			running_distance+=(current_ray_origin.distance(hit_point))
			
			if path_array.index(entity)
				hit_entity.push(true)
				item=nil
			else
				current_ray_origin=hit_point
				new_ray=[current_ray_origin,face_normal]
				item=model.raytest(new_ray)
			end
			
		end
		
		if running_distance==0.0
			hit_entity.push(false)
		end
		distance_to_entity.push(running_distance)
	}
	if hit_entity.index(true)
		return distance_to_entity.max.to_f+0.1
	else
		return nil
	end
else
	return nil
end
end
##
def get_closest_face_to_point(faces,point)
trans=self.transformation
distance_to_point=[]
faces.each_index {|index|
	verts=faces[index].vertices
	box=Geom::BoundingBox.new
	face_pts=verts.collect {|v| v.position}
	box.add(face_pts)
	face_center=box.center
	distance_to_point.push(face_center.vector_to(point).length)
}
min_distance=distance_to_point.min
min_index=distance_to_point.index(min_distance)
return faces[min_index]
end
####calculates area of an array of points - points must form a loop in counter clockwise order
def polygon_area(pts,transformation=Geom::Transformation.new)
pts=pts.collect {|p| p.transform(transformation)}  #transform the points
pts.push(pts.first)  #close the loop
sum=0.0
num_points=pts.length
(num_points-1).times {|i|
	sum+=(pts[i].x*pts[i+1].y)-(pts[i+1].x*pts[i].y)
}
return (0.5*sum)
end
##
def get_normal_of_vertices(verts)
edges=verts[0].edges
perp_edge=edges.find {|edge| verts.index(edge.other_vertex(verts[0]))==nil}
edge_vector=perp_edge.other_vertex(verts[0]).position.vector_to(verts[0].position)
return edge_vector.normalize
end
###
def transform_by_placement_point(points,placement_point)
box=Geom::BoundingBox.new
box.add(points)
min=box.min
max=box.max
cent=box.center
case placement_point
when 1 #top right
	pp=[min.x,max.y,0.0]
when 2 #top middle
	pp=[cent.x,max.y,0.0]
when 3 #top right
	pp=[max.x,max.y,0.0]
when 4 #mid left
	pp=[min.x,cent.y,0.0]
when 5 #centroid
	pp=cent
when 6 #mid right
	pp=[max.x,cent.y,0.0]
when 7 #bot left
	pp=min
when 8 #bot middle
	pp=[cent.x,min.y,0.0]
when 9 #bot right
	pp=[max.x,min.y,0.0]
end
trans=Geom::Transformation.new(pp)
trans.invert!
return trans
end
#########
def transform_by_rotation(points,rotation)
rot_radians=rotation.degrees
rot_trans=Geom::Transformation.rotation([0,0,0],[0,0,1],rot_radians)
return rot_trans	
end
#########
def transform_by_offset(points,x_offset,y_offset)
offset_trans=Geom::Transformation.translation(Geom::Vector3d.new(x_offset,y_offset,0.0))
return offset_trans	
end
####
#transform points by scaling
def transform_by_size(x_size,y_size,x_scale,y_scale)
scale_trans=Geom::Transformation.scaling(x_scale,y_scale,1.0)
return scale_trans
end
################################################
#transform points by mirroring
def transform_by_mirror(points)
mirror_trans=Geom::Transformation.scaling(-1,1,1)
return mirror_trans	
end
##
def projected_face_area(f,vector)
f_normal=f.normal
projected_area=f.area*(f_normal.dot(vector)).abs
return projected_area
end
end #module Geom
module WebDialogHelper
def setCheckbox(id,checked)
	cmd="document.getElementById('#{id}').checked=#{checked}"
	@dlg.execute_script(cmd)
end
def setValue(id,value,dlg=@dlg)   #sets the value of an input field in the web dialog
	new_value=value.to_s
	cmd="document.getElementById('#{id}').value=\"#{new_value}\""
	dlg.execute_script(cmd)
end
def setSelectWidget(id,value)
setValue(id,value)
cmd="$('##{id}').selectmenu( 'refresh' );"
@dlg.execute_script(cmd)
end
##
def refreshMaterialsList(id,include_default_mat=true)
id="materialName"
mats=Sketchup.active_model.materials
cmd="clearList(\"#{id}\")"
@dlg.execute_script(cmd)
material_names=mats.collect {|mat| mat.name}
material_names=material_names.sort
material_names.unshift("Default") if include_default_mat
material_names.each {|name|
	cmd="createOption(\"#{id}\",\"#{name}\")"
	@dlg.execute_script(cmd)
}
end
##
def refreshLayersList(id,include_default_layer=true)
layers=Sketchup.active_model.layers
cmd="clearList(\"#{id}\")"
@dlg.execute_script(cmd)
layers.each {|layer|
	if !include_default_layer
		next if layer==layers[0]
	end
	cmd="createOption(\"#{id}\",\"#{layer.name}\")"
	@dlg.execute_script(cmd)
}
end
##
def store_dlg_size(dlg_name,width,height)
Sketchup.write_default(dlg_name,"dlgwidth",width)
Sketchup.write_default(dlg_name,"dlgheight",height)
end
##
def get_dlg_size(dlg_name)
width=Sketchup.read_default(dlg_name,"dlgwidth")
height=Sketchup.read_default(dlg_name,"dlgheight")
return [width.to_i,height.to_i]
end
end #webdialog helper
end #module PB
end #module DM
