require 'sketchup.rb'
module DM
module ProfileBuilder2
class AppObserver < Sketchup::AppObserver
def expectsStartupModelNotifications()
   return true
end
def onNewModel(model)
   start_pb_observers()
end
def onOpenModel(model)
	start_pb_observers()
end
def start_pb_observers()
ProfileBuilderMTO.close()
FenceBuilderTool.close()
Sketchup.active_model.tools.add_observer(AxesToolObserver.new)
end
end #class
Sketchup.add_observer(AppObserver.new)
##
class SelectionObserver < Sketchup::SelectionObserver
def initialize(&block)
@callback=block
super
end
def onSelectionBulkChange(selection)	
	@callback.call
	#p "changed"
end
def onSelectionCleared(selection)
   @callback.call
   #p "cleared"
end
end #class selection observer
##
class ModelObserver < Sketchup::ModelObserver
def initialize(&block)
@callback=block
super
end
##
def onActivePathChanged(model)
	@callback.call
end
end #class
##
class EditingFenceObserver < Sketchup::InstanceObserver
   def onClose(inst)
   
	timer_id=UI.start_timer(0.01,false){  #a timer is required due to an obscure sketchup bug where the start_operation method fails to properly create the undo stack inside the observer
	Sketchup.active_model.start_operation("Modify Fence Path",true)
	
	begin
		ents=inst.class==Sketchup::Group ? inst.entities : inst.definition.entities
		edges=ents.grep(Sketchup::Edge)
		new_chain=Chain.new
		status=new_chain.set(edges)
		
		if status
			p1=new_chain.first  #get the first point in the path
			p2=new_chain.last   #get the end point of the path
			vec1=Geom::Vector3d.new(p1.x,p1.y,p1.z)
			vec2=Geom::Vector3d.new(p2.x,p2.y,p2.z)
			
			if vec1.length>vec2.length
				new_chain.reverse!
			end
			a=Assembly.get(inst)
		else
			UI.stop_timer(timer_id)
			UI.messagebox("Unable to edit Assembly path. Ensure path is valid.")
		end
		a.set_chain(new_chain)
		a.draw()
		
	rescue
		UI.stop_timer(timer_id)
		Sketchup.active_model.abort_operation
		raise
	end
	inst.remove_observer(self)
	Sketchup.active_model.commit_operation
	}
   end
end
##
class EditingProfileMemberObserver < Sketchup::InstanceObserver
	
	def initialize(pm)
	
	@epm=pm
	super()
	
	end
	
	
	def onClose(inst)
   
   timer_id=UI.start_timer(0.01,false){  #a timer is required due to an obscure sketchup bug where the start_operation method fails to properly create the undo stack inside the observer
	#epm=ProfileBuilderTool.editing_profile_member   #get the profile member currently being edited
	Sketchup.active_model.start_operation("Modify Member Path",true)
	
	begin
		ents=inst.class==Sketchup::Group ? inst.entities : inst.definition.entities
		edges=ents.grep(Sketchup::Edge)
		new_chain=Chain.new
		status=new_chain.set(edges)
		
		if status
			p1=new_chain.first  #get the first point in the path
			p2=new_chain.last   #get the end point of the path
			vec1=Geom::Vector3d.new(p1.x,p1.y,p1.z)
			vec2=Geom::Vector3d.new(p2.x,p2.y,p2.z)
			
			#determine which is the desired start of the path based on the distance from the origin  (the profile member origin is always at the start point of the path)
			if vec1.length>vec2.length
				new_chain.reverse!
			end
				
			@epm.set_chain(new_chain)#store the new chain
		else
			UI.stop_timer(timer_id)
			UI.messagebox("Unable to edit ProfileMember path. Ensure path is valid.")
		end
		@epm.draw()
		
	rescue
		UI.stop_timer(timer_id)
		UI.messagebox("Unable to edit ProfileMember path. Ensure path is valid.")
		#Sketchup.active_model.abort_operation
		raise
	end
	inst.remove_observer(self)
	Sketchup.active_model.commit_operation
	}
   end
end
######
class LayersObserver < Sketchup::LayersObserver
#store the calling tool in this class variable
def caller=(pb_tool)
	@caller=pb_tool
end
def refresh_caller()
@caller.refreshLayersList("layerName",true) if @caller
@caller.setLayerName() if @caller
end
def onLayerAdded(layers, layer)
	refresh_caller()
end
def onLayerRemoved(layers, layer)
	refresh_caller()
end
def onRemoveAllLayers(layers)
	refresh_caller()
end
end  #end layers observer
####
class MaterialsObserver < Sketchup::MaterialsObserver
#store the calling tool in this class variable
def caller=(pb_tool)
	@caller=pb_tool
end
def refresh_caller()
@caller.refreshMaterialsList("materialName",true) if @caller
@caller.setMaterialName() if @caller
end
def onMaterialAdd(materials, material)
	refresh_caller()
end
def onMaterialRemove(materials, material)
	refresh_caller()
end
end

##
class AxesToolObserver < Sketchup::ToolsObserver
def onActiveToolChanged(tools, tool_name, tool_id)
	
case tool_name.to_s
when "SketchCSTool"
	active_path=Sketchup.active_model.active_path
	e=active_path[-1] if active_path
	if Classify.profile_member?(e)
		start_observer(e)
		@using_sketchCS=true
	end
when "ComponentCSTool"
	e=Sketchup.active_model.selection.first
	if Classify.profile_member?(e)
		@inst=e
		@current_trans=@inst.transformation
		@using_componentCS=true
	end
else
	if @using_sketchCS
		@using_sketchCS=false
	end
	if @using_componentCS
		if @inst and @inst.valid? 
			transform_stored_path()
		end
		@using_componentCS=false
	end
end
end
##
def transform_stored_path()
new_trans=@inst.transformation
pb=ProfileMember.new(@inst)
chain=pb.chain
chain.transform!(@current_trans)
chain.transform!(new_trans.inverse)
pb.set_chain(chain)
@inst=nil
end
##
def start_observer(e)
@inst=e
t=Sketchup.active_model.edit_transform
@acio=AxesChangeInstanceObserver.new(t)
@inst.add_observer(@acio)
end
 
end  #class
 
##
class AxesChangeInstanceObserver < Sketchup::InstanceObserver
def initialize(trans)
@trans=trans
super()
end
##
def onClose(inst)
	
	timer_id=UI.start_timer(0.2,false) {
		pb=ProfileMember.new(inst)
		new_trans=inst.transformation
		chain=pb.chain
		chain.transform!(@current_trans)
		chain.transform!(new_trans.inverse)
		pb.set_chain(chain)
		inst.remove_observer(self)
	}
	
end
end  #class

end #module PB
end #module DM
