require 'sketchup.rb'
module DM
module ProfileBuilder2
class Profile 
include Geometry
attr_reader :placement_point,:mirror,:rotation,:x_offset,:y_offset,:smooth_angle, :material_name, :layer_name
attr_writer :placement_point,:mirror,:rotation,:x_offset,:y_offset,:smooth_angle, :material_name, :layer_name

public
#create a new profile object from either a string or a face.  an optional name can be specified
def initialize(object,name="")
	@name=name          #the name of the profile
	@outer_loop=[]
	@holes=[]
	@placement_point=5
	@mirror=false
	@rotation=0.0
	@x_offset=0.0
	@y_offset=0.0
	@smooth_angle=30.0
	@file_path=nil
	@dimension=2
	@material_name="Default"
	@layer_name=Sketchup.active_model.layers[0].name
	@dict="ProfileBuilder"
	@key="Properties"
	@x_scale=1.0
	@y_scale=1.0
	
	if object.class==String   #create a new profile object from the string representation of the object
		load_from_string(object)
	elsif object.class==Sketchup::Face or object.class==Array  #create a new profile object from any sketchup face entity
		self.profile=(object)
	elsif object.class==Profile
		return Profile.new(object.to_s)
	end
end
#
def x_scale=(value)
	@x_scale=value.to_f
end
##
def x_scale
	return @x_scale
end
##
def y_scale=(value)
	@y_scale=value.to_f
end
##
def y_scale
	return @y_scale
end
##
def save_skp(folder,file_name=nil)
file_name=self.name if file_name==nil
cam=Sketchup.active_model.active_view.camera
eye=cam.eye
target=cam.target
up=cam.up
temp_target=Geom::Point3d.new(0,0,0)
temp_up=Geom::Vector3d.new(0,1,0)
temp_eye=Geom::Point3d.new(0,0,100)
cam.set(temp_eye,temp_target,temp_up)
Sketchup.active_model.start_operation("save_profile",true)
model=Sketchup.active_model
defns=model.definitions
profile_defn=defns.add(file_name)
ents=profile_defn.entities
self.draw(ents,true)
profile_defn.set_attribute(@dict,@key,self.to_s)
path=File.join(folder,file_name)
path+=".skp"
profile_defn.save_as(path)
Sketchup.active_model.abort_operation
cam.set(eye,target,up)
end
##
def load_from_skp(skp)
begin
	Sketchup.active_model.start_operation("Import Profile",true)
	su_defn=import_component(skp)
	load_from_defn(su_defn)
	pb_material=PBMaterial.new(self.material_name)
	Sketchup.active_model.abort_operation
	
	Sketchup.active_model.start_operation("Import Profile",true)
	profile_material=pb_material.to_su_material()
	if profile_material
		self.material_name=profile_material.name
	else
		self.material_name="Default"
	end
	l_name=self.layer_name
	if l_name
		layers=Sketchup.active_model.layers
		if !layers[l_name]
			layers.add(l_name)
		end
	end
	Sketchup.active_model.commit_operation
rescue IOError
	Sketchup.active_model.abort_operation
	ProfileBuilder2.save_last_profile_path("")
	UI.messagebox("Invalid SketchUp Component File")
	return false
rescue InvalidProfileSKP => err
	ProfileBuilder2.save_last_profile_path("")
	Sketchup.active_model.abort_operation
	UI.messagebox(err.message)
	return false
end
return true
end
##
def draw(ents,use_pb_default=false)
path_extruder=PathExtruder.new
path_extruder.profile=self
path_extruder.path=[[0,0,0],[0,0,-1]]
mats=Sketchup.active_model.materials
mat=mats[@material_name]
if use_pb_default
	def_mat=mats[DEFAULT_MAT_NAME]
	if def_mat==nil
		def_mat=mats.add(DEFAULT_MAT_NAME) if def_mat==nil
		def_mat.color="gray"
	end
else
	def_mat=nil
end
if self.is_2d?
	face=path_extruder.draw_start_cap_face(ents)
	#face.reverse!
	if mat
		face.material=mat
	else	
		face.material=def_mat
	end
else
	edges=path_extruder.draw_start_cap_polyline(ents)
	edges.each {|e| e.material=mat} if mat
end
end
##
def clone
	cloned_profile=Profile.new(self.to_s)
	cloned_profile.x_scale=self.x_scale
	cloned_profile.y_scale=self.y_scale
	return cloned_profile
end
####sets the profile to an existing face or array of points (for 1D profiles)
def profile=(object)
	
	auto_reverse=false
	if object.class==Sketchup::Face
			su_face=object
			normal=su_face.normal
			box=su_face.bounds
			origin=box.min
			if origin.z==0.0 and normal.samedirection?(Z_AXIS.reverse)
				auto_reverse=true
				normal.reverse!
			end
			trans=Geom::Transformation.new(origin, normal)
			trans.invert!
			@holes=[]
			loops=su_face.loops
			loops.each {|loop|
				verts=loop.vertices
				verts.reverse! if auto_reverse
				if loop.outer?
					@outer_loop=verts.collect {|v| v.position.transform(trans)}
				else
					hole=[]
					hole=verts.collect {|v| v.position.transform(trans)}
					@holes.push(hole)
				end
			}
	else
			normal=Geom::Vector3d.new(0,0,1)			
			box=Geom::BoundingBox.new
			object.each {|p| box.add(p)}
			origin=box.min
			trans=Geom::Transformation.new(origin, normal)
			trans.invert!
			@outer_loop=object.collect {|p| p.transform(trans)}
			@dimension=1
	end
		
end

def set_from_profile_member(pm)
pm_profile=pm.profile
self.points=pm_profile.points
@holes=pm_profile.holes
@dimension=pm_profile.dimension
self.name=pm_profile.name
self.placement_point=pm.placement_point
self.rotation=pm.rotation
self.x_offset=pm.x_offset
self.y_offset=pm.y_offset
self.smooth_angle=pm.smooth_angle
self.mirror=pm.mirror
self.x_scale=pm.x_scale
self.y_scale=pm.y_scale
self.material_name=pm.material_name
self.layer_name=pm.layer_name
end
#allows profiles to be compared by name for sorting
def <=>(profile)
	return self.name<=>(profile.name)
end
####################sets the name of the profile
def name=(new_name)
	illegal="'\" |"  #characters not allowed in profile name
	@name=new_name.delete("\\")
	@name=@name.delete(illegal)  #for some reason, it doesn't work to combine this and the previous line
end
####
def name
	@name
end
##############get the outer loop of the profile - returns an array of point3d objects forming the outer perimiter of the profile
def points
	return @outer_loop.dup	
end
def outer_loop
return self.points
end
##############sets the outer loop of the profile
def points=(pts)
@outer_loop=pts
end
#####add a hole to the profile
def add_hole(pts)  #points is an array of point3d objects
@holes.push(pts)
end
#############get the array of holes
def holes
	return @holes
end
#determine if we have a 1D profile
def dimension
return @dimension
end
def is_2d?
return @dimension==2 ? true : false
end
def is_1d?
return @dimension==1 ? true : false
end
############calculates the area of a profile without the need to create temporary geometry
def area(trans=Geom::Transformation.new)
outer_loop=self.points.collect {|p| p.transform(trans)}  #get the transformed profile
#now transform all the holes
t_holes=[]
self.holes.each {|hole|
	hole=hole.collect {|p| p.transform(trans)}
	t_holes.push(hole)
}
#get the normal of the face
v1=outer_loop[1]-outer_loop[0]
v2=outer_loop[2]-outer_loop[1]
normal=v1.cross(v2).normalize! 
#now rotate the profile to ensure that all points lie on the X-Y plane
origin=Geom::Point3d.new(0,0,0)
trans2=Geom::Transformation.new(origin,normal)
trans2.invert!
outer_loop_area=self.polygon_area(outer_loop,trans2)
hole_area=0.0
t_holes.each {|hole| hole_area+=self.polygon_area(hole,trans2)}
return outer_loop_area+hole_area   #hole area has opposite sign
end
def default_width
box=Geom::BoundingBox.new
box.add(@outer_loop)
return box.width
end
##
def default_height
box=Geom::BoundingBox.new
box.add(@outer_loop)
return box.height
end
################returns the width of the profile
def width
return default_width*x_scale
end
################returns the height of the profile
def height
return default_height*y_scale
end
#
def scaled?
scaled=false
if self.x_scale!=1.0 or self.y_scale!=1.0
	scaled=true
end
return scaled
end
###
def get_transformed_loop(profile_pts)
t1=transform_by_placement_point(@outer_loop,@placement_point) 
profile_pts=profile_pts.collect {|p| p.transform(t1)}
t=transform_by_size(width,height,@x_scale,@y_scale)
profile_pts=profile_pts.collect {|p| p.transform(t)}
t2=transform_by_rotation(profile_pts,@rotation)
profile_pts=profile_pts.collect {|p| p.transform(t2)}
t4=transform_by_offset(profile_pts,@x_offset,@y_offset)
profile_pts=profile_pts.collect {|p| p.transform(t4)}
if @mirror
	t3=transform_by_mirror(profile_pts)
	profile_pts=profile_pts.collect {|p| p.transform(t3)}
	profile_pts.reverse! #if is_hole
end
return profile_pts
end

#################creates a string representation of the profile in order to store in an attribute dictionary or written to a profile library file
def to_s
	points=[]
	if @dimension==1
		#@outer_loop.push(@outer_loop[0])
	end
	@outer_loop.each {|p|
		points.push("#{p.x.to_f} #{p.y.to_f}")
	}
	points=points.join(" ")
	string="#{@name} #{points}"
	@holes.each {|hole|
		hole_pts=[]
		hole.each {|p|
			hole_pts.push("#{p.x.to_f} #{p.y.to_f}")
		}
		hole_pts=hole_pts.join(" ")
		hole_string="|#{hole_pts}"
		string+=hole_string
	}
	string+="|extended|#{@placement_point}|#{@mirror.to_s}|#{@rotation}|#{@x_offset}|#{@y_offset}|#{@smooth_angle}|#{@x_scale}|#{@y_scale}|#{@dimension}|#{@material_name}|#{@layer_name}"
	
	string+="\n"
	return string
end

#####################################
private
	class InvalidProfileSKP < RuntimeError
	end	
	
	##
	def import_component(skp)
	
		defns=Sketchup.active_model.definitions
		su_defn=defns.load(skp)
		return su_defn
	
	end
	
	##
	def add_point(p)
		@outer_loop.push(p)
	end
	##
	def load_from_defn(su_defn)
	f=get_profile_face(su_defn)
	valid_path=get_valid_path(su_defn)
	if f or valid_path
		initialize(nil)
		load_attributes_from_defn(su_defn)
		if f
			self.profile=f
			#if f.material and self.material_name=="Default"
				#self.material_name=f.material.name
			#end
		elsif valid_path
			edges=valid_path
			verts=edges.collect {|edge| edge.vertices}
			verts.flatten!
			start_vert=verts.find {|v| v.get_attribute(@dict,'polyline_index')==0}
			pts_chain=Chain.new
			pts_chain.set(edges,start_vert)
			self.profile=pts_chain.path
			#mats=edges.collect {|edge| edge.material}
			#mats.delete(nil)
			#self.material_name=mats[0].name unless mats.empty?
		end
		if self.name==""
			self.name=File.basename(su_defn.path,".skp") if self.name==""
		end
	else
		raise InvalidProfileSKP, "Component is not a valid Profile.  Ensure that the component only contains one face or polyline.",caller
	end
	
	return true
	end
	
	##
	def get_profile_face(su_defn)
	
	f=nil
	ents=su_defn.entities
	faces=ents.grep(Sketchup::Face)
	if faces.length==1
		f=faces[0]
	end
	
	return f
		
	end
	
	##
	def get_valid_path(su_defn)
	
	valid_path_edges=nil
	ents=su_defn.entities
	edges=ents.grep(Sketchup::Edge)
	if edges.length>0
		pts_chain=Chain.new
		valid_path=pts_chain.set(edges)
	end
	
	if valid_path
		valid_path_edges=edges
	end
	
	return valid_path_edges
	
	end
	
	##
	def load_attributes_from_defn(su_defn)
	
	dicts=su_defn.attribute_dictionaries
	if dicts and dicts[@dict]
		profile_properties=su_defn.get_attribute(@dict,@key,nil)
		if profile_properties
			load_from_string(profile_properties)
		end
	end
		
	end
	
	##
	def load_from_string(string)
	props=string.split("|extended|")
	base_props_string=props[0]  #these are the initial properties saved in PB 1.0
	base_props=base_props_string.split("|")
	base_props.each_index {|i|
		case i
		when 0 #the outer loop
			load_outer_loop_from_string(base_props[i])
		else
			load_hole_from_string(base_props[i])
		end
	}
	if props.length>1
		load_extended_properties_from_string(props[1])
	end
	end
	##
	def load_outer_loop_from_string(outer_loop)
		parse_points=outer_loop.split(/\s+/)
		len=parse_points.length
		num_points=((len-1).to_f/2.0).to_i
		@name=parse_points[0]
		j=1
		while (j<len)
			p=Geom::Point3d.new(parse_points[j].to_f,parse_points[j+1].to_f,0)
			add_point(p)
			j+=2
		end
	end
	def load_hole_from_string(hole)
		hole_pts=[]
		parse_points=hole.split(/\s+/)
		len=parse_points.length
		num_points=((len).to_f/2.0).to_i
		j=0
		
		while (j<len)
			p=Geom::Point3d.new(parse_points[j].to_f,parse_points[j+1].to_f,0)
			hole_pts.push(p)
			j+=2
		end
		@holes.push(hole_pts)
	end
	def load_extended_properties_from_string(string)
	
	string=string.delete("\n")
	props=string.split("|")
	@placement_point=props[0].to_i
	@mirror=props[1]=="true" ? true : false
	@rotation=props[2].to_f
	@x_offset=props[3].to_f
	@y_offset=props[4].to_f
	@smooth_angle=props[5].to_f
	@x_scale=1.0 #props[6].to_f
	@y_scale=1.0 #props[7].to_f
	@dimension=props[8].to_i
	@material_name=props[9]==nil ? "Default" : props[9]
	@layer_name=props[10]==nil ? Sketchup.active_model.layers[0].name : props[10]
	end

end #class profile end
end #module PB
end #module DM
