# Name :          Mirror
# Description :   Mirrors Selection at Point, Line or Plane
# Author[s] :     From an original idea by Frank Wiesner: 
#		          but from v2.6 onwards it's been fully TIG tweaked, 
#                 and completely updated and rewritten...###
# Usage :         1. Make Selection (Any type[s] of Entites are allowed)###
#                 2. Select 'Mirror Selection' from Plugins Menu 
#					 or from the equivalent on the Right-Click Context-Menu
#                    or using your favorite ShortCut Key [e.g.Shift+R] ###
#                    or from the 'Mirror' Toolbar [this toolbar will be  
#                    available for activation from the View>Toolbars 
#                    Menu IF the button icon image MI.png is in the 
#                    same folder as Mirror.rb file (usually Plugins)] ###v3.2
#                 3. If there's no Selection then dialog informs you ###
#                 4. Pick a Point and RETURN to Mirror at the Point
#                 5. OR Pick a Second Point and RETURN to Mirror at the Line
#                 6. OR Pick a Third Point to Mirror at the Plane [usual]
#                 7. Final Option is Erase Original ? Yes/No ###
#					 If Yes then Highlighting of Selection passes to Copies
# Type :      	Script - Mirror.rb
# History: 
# 3.8  09 Aug 2012 ### TIG, locked objects are no longer mirrored, code tidying.
# 3.7  05 Feb 2012 ### TIG, mirroring glued instances now [re]glues them.
# 3.6  13 Dec 2010 ### TIG, group.copy methods recast to avoid glitches.
# 3.5  10 Nov 2010 ### TIG, context-menu re-added.
# 3.4  9 Nov 2010 ### TIG, Recoded to avoid clashes and crashes...
# 3.3  3 Oct 2010 ### TIG, togglewindows catch for Outliner added and set zipped.
# 3.2  19 Feb  2010 ### TIG, optional toolbar added
#      [this doesn't load is the MI.png file is NOT found].
# 3.1  21 Feb  2007 ### TIG make_unique tries to avoid rare bugsplats.
# 3.0  05 Aug  2006 ### TIG, Ending rewritten to avoid rare bugsplats.
# 2.9  03 Aug  2006 ### TIG, Start:commit loops to avoid rare bugsplats.
# 2.8  26 July 2006 ### TIG, re-tweaked - Rewritten to be MUCH simpler.
#      Mirroring of ANY Entity Selection works properly.
# 2.7  23 July 2006 ### TIG's second tweak - Rewritten generally.
#      Now processes Multiple selections of Groups and/or 
#      Components, but warns if any 'loose' (UnGrouped) 
#      Faces in Selection as they might lose any RePostioned 
#      Texture data, interfere etc.  
# 2.6  22 July 2006 ### TIG first tweak ### - Context menu added IF ONE 
#      group/compo.  
#      Warns about selection IF NOT ONE group/compo.  
#      Materials now preserved in mirrored copy BUT you will 
#      need to group selection to preserve material positioning 
#      - although this is define-able it is not find-able from 
#      the orignal face info !  
#      Final option to Erase of Original Group/Compo.
# 2.5  (8.Jul.2oo5)  - bugfix: faces with holes reduce now to a single face
#      - major code rewrite (dropped @clondedObject hash, heavy 
#      use of groups during construction,...
#      - more redundant constructions, but seems to be more robust
#      - WARNING: POLYGONS DO NOT WORK VERY WELL
# 2.4  (5.Jul.2oo5)  - all unnecessary edges created by the add_faces_from_mesh() 
#      method are erased! faces with holes mirror now perfectly.
# 2.3  (4.Jul.2oo5)  - fixed "mirror at plane" error
# 2.2  (2.Jul.2oo5)  - works in group editting mode - prevents picking of identical points
# 2.1  (1.Jul.2oo5)  - fixed "undo group" bug - code cleanup
# 2.0  (30.Jun.2oo5) - new tool-like interface
# 1.4  (29.Jun.2oo5) - arc bug fixed - code simplified (clone_selection() removed)
# 1.3  (24.Jun.2oo5) - can handle faces with holes in it (although more edges are drawn than nessesary) - correct orientation of mirrored faces - supports arcs (at least works for circles), curves and groups
# 1.2  (20.Jun.2oo5) - axis and origin selection algorithm does not rely on tool tips any more.
# 1.1  (19.Jun.2oo5) - fix coplanar point bug - better tooltips - supports mirror at lines (edges, axis, construction lines) - mirror-at-plane code simplified
# 1.o  (10.Jun.2oo5) - first version
################################################################
require 'sketchup.rb'
#####################


#####################

class MirrorTool

#### BugSplat limiter: Outliner rollup tool after Jim's code - only for Windows...
if [RUBY_PLATFORM].grep(/mswin/)==[RUBY_PLATFORM] && File.exist?(File.join(File.dirname(__FILE__),"toggleWindows.rb")) ### = a Windows machine and script there...
  require 'toggleWindows.rb'
  def hide_Outliner_MI()
    @the_Outliner_was_Open_MI=!isRolledUp("Outliner")
    toggleRollUp("Outliner")if @the_Outliner_was_Open_MI
  end
  def restore_Outliner_MI()
    toggleRollUp("Outliner")if @the_Outliner_was_Open_MI
  end
end#if

    def initialize
        @MIRROR_AT_POINT = 1
        @MIRROR_AT_LINE  = 2
        @MIRROR_AT_PLANE = 3
    end
    
    def reset()
        @pts = []
        @state = 0
        @transformationType = nil
        @trans = nil
        @point = nil
        @line = nil
        @linev = nil
        @plane = nil
        @planeNormal = nil
        @pointOnPlane = nil
        @ip = Sketchup::InputPoint.new
        @ip1 = Sketchup::InputPoint.new
        @ip2 = Sketchup::InputPoint.new
        @ip3 = Sketchup::InputPoint.new
        @msg = "提醒： 请指定镜像的第1个点"
        Sketchup::set_status_text(@msg)###
        @drawn = false
    end
    
    def activate
        @model=Sketchup.active_model
        @ss=@model.selection
		@locked=[]
		@ss.each{|e|
		  next unless e.is_a?(Sketchup::Group) || e.is_a?(Sketchup::ComponentInstance)
		  @locked << e if e.locked?
		}
		@ss.remove(@locked) if @locked[0]
        if @ss.empty?
			@ss.add(@locked) if @locked[0]
            Sketchup::set_status_text("提醒： 没有选择有效的物体!")###
    		UI.messagebox("使用镜像工具前请选择有效的物体！")###v2.8
            Sketchup.send_action("selectSelectionTool:")
            return nil
        end
        self.hide_Outliner_MI() if [RUBY_PLATFORM].grep(/mswin/)==[RUBY_PLATFORM] && File.exist?(File.join(File.dirname(__FILE__),"toggleWindows.rb"))
        self.reset()
    end
    
    def deactivate(view)
        view.invalidate if @drawn
        @ip1 = nil
        @ip2 = nil
        @ip3 = nil
        self.restore_Outliner_MI() if [RUBY_PLATFORM].grep(/mswin/)==[RUBY_PLATFORM] && File.exist?(File.join(File.dirname(__FILE__),"toggleWindows.rb"))
    end
    
    def resume(view=nil)
        Sketchup::set_status_text(@msg)
    end
    
    def getExtents
      bb=Geom::BoundingBox.new
      case @state
       when 0 # We are getting the first point
         if @ip.valid? && @ip.display?
            bb.add(@ip.position)
         end
       when 1
         bb.add(@pts[0]) if @pts[0]
       when 2
         bb.add(@pts[1]) if @pts[1]
      end
      return bb
    end
    
    def onMouseMove(flags, x, y, view)
        case @state
        when 0 # getting the first end point
            @ip.pick(view, x, y)
            if @ip.valid? && @ip != @ip1
                @ip1.copy!(@ip)
            end
            view.invalidate
            view.tooltip = @ip.tooltip if @ip.valid?
        when 1 # getting the second end point
            @ip.pick(view, x, y, @ip1)
            if @ip.valid? && @ip != @ip2
                @ip2.copy!(@ip)
                @pts[1] = @ip2.position
            end
            view.invalidate
            view.tooltip = @ip.tooltip if @ip.valid?
        when 2 # getting the third end point
            @ip.pick(view, x, y, @ip2)
            if @ip.valid? && @ip != @ip3
                @ip3.copy!(@ip)### v3.4
                @pts[1] = @ip3.position
            end
            view.invalidate
            view.tooltip = @ip.tooltip if @ip.valid?
        end
    end
    
    def onLButtonDown(flags, x, y, view)
        @ip.pick(view, x, y)
        if @ip.valid?
            case @state
            when 0
                if @ip.valid?
                    @pts[0] = @ip.position
                    @msg = "提醒：按“回车键” 以“对称点”镜像，或者继续指定镜像的第2个点"
                    Sketchup::set_status_text(@msg)
                    @state = 1
                end
            when 1
                if @ip.valid? && @ip != @ip1
                    @pts[1] = @ip.position
                    @state = 2
                    @msg = "提醒：按“回车键” 通过“对称轴”镜像，或者继续指定镜像的第3个点"
                    Sketchup::set_status_text(@msg)
                end
            when 2
                if @ip.valid? && @ip != @ip1 && @ip != @ip2
                    @pts[2] = @ip.position
                    @state = 3
                    view.invalidate
                    self.mirror()
                end
            end
        end
    end
    
    def onCancel(flag, view)
        view.invalidate if @drawn
        self.reset()
    end
    
    def draw(view)
	    return unless @ip
        if @ip.valid? && @ip.display?
            @ip.draw(view)
            @drawn = true
        end
        if @state == 1
            view.set_color_from_line(@ip1, @ip)
            view.draw(GL_LINE_STRIP, @ip1.position, @ip.position)
            @drawn = true
        elsif @state == 2
            view.drawing_color = "gray"
            view.draw(GL_LINE_STRIP, @ip1.position, @ip2.position)
            view.set_color_from_line(@ip2, @ip)
            view.draw(GL_LINE_STRIP, @ip2.position, @ip.position)
            @drawn = true
        end
    end
    
    def onKeyDown(key, repeat, flags, view)
        if key == CONSTRAIN_MODIFIER_KEY && repeat == 1
            @shift_down_time = Time.now
            if view.inference_locked?
                view.lock_inference
            elsif @ip.valid?
                view.lock_inference(@ip)### v3.4
            end
        end
    end
    
    def onKeyUp(key, repeat, flags, view)
        if key == CONSTRAIN_MODIFIER_KEY && view.inference_locked? && (Time.now - @shift_down_time) > 0.5
            view.lock_inference
        end
        if key == 13
            self.mirror()
            self.reset()
        end
    end
    
    def mirror()
        ###
       case @state
        when 1
            @transformationType = @MIRROR_AT_POINT
            @point = @ip1.position.clone
            @trans = Geom::Transformation.scaling(@point, -1.0)
            @model.start_operation("通过点镜像")
        when 2
            @transformationType = @MIRROR_AT_LINE
            @line = [@ip1.position, @ip2.position] 
            @linev = Geom::Vector3d.new(@ip2.position.x-@ip1.position.x, @ip2.position.y-@ip1.position.y, @ip2.position.z-@ip1.position.z)
            @model.start_operation("通过线镜像")
        when 3
            @transformationType = @MIRROR_AT_PLANE
            @plane = Geom.fit_plane_to_points(@ip1.position, @ip2.position, @ip3.position) 
            @planeNormal = Geom::Vector3d.new(@plane[0], @plane[1], @plane[2])
            @planeNormal.normalize!
            @pointOnPlane = @ip1.position
            @model.start_operation("通过面镜像")
       end
       @model.active_view.invalidate
    	### fix group glitch v3.1
      def group_miner(ents)
        ents.each{|e|
          next if not e.class==Sketchup::Group### v3.4
          if e.entities.parent.instances[1]### v3.4
            begin ### v3.4
              e.make_unique
              self.group_miner(e.entities.to_a)
            rescue
              ###
            end ### v3.4
          end#if
        }#end each
      end#def
      sents=@ss.to_a### v3.4
      ents=sents[0].parent.entities
      @ss.clear
      self.group_miner(sents)
	  ###
	  ### get ONLY glued instances
	  @is2ds=[]
	  sents.each{|e|
	    next unless e.class==Sketchup::ComponentInstance
		next unless e.definition.behavior.is2d?
		@is2ds << e.definition
	  }
	  @is2ds.uniq!
      ###
	  ### make group
      copy_group=ents.add_group(sents)
      ### 
      ###new_group = copy_group.copy          ###v3.4/v3.6
      gdef=copy_group.entities.parent ###3.6
      gtr=copy_group.transformation ###3.6
      new_group=ents.add_instance(gdef, gtr)###3.6
      self.group_miner(new_group.entities.to_a) ###v3.4
      ###
        ###nents=new_group.entities.to_a### v3.4
        new_center = new_group.bounds.center.clone
        mirror_point(new_center)
        t = nil
        t = Geom::Transformation.scaling(new_center,-1,-1,-1)if @state==1
        t = Geom::Transformation.rotation(new_center,@linev,180.degrees)if @state==2
        tt=nil
        if @state==3
           xxx=@planeNormal.normalize.to_a[0].abs*-1
           yyy=@planeNormal.normalize.to_a[1].abs*-1
           zzz=@planeNormal.normalize.to_a[2].abs*-1
           xxx=1 if xxx>0
           yyy=1 if yyy>0
           zzz=1 if zzz>0
           xxx=-1 if xxx<=0 
           yyy=-1 if yyy<=0
           zzz=-1 if zzz<=0
           t = Geom::Transformation.scaling(new_center,xxx,yyy,zzz)
           tt= Geom::Transformation.rotation(new_center,@planeNormal,180.degrees)
        end#if
        new_group.transform!(t)if t###mirror
        new_group.transform!(tt)if tt###
        transVec = Geom::Vector3d.new(new_center.x-new_group.bounds.center.x, new_center.y-new_group.bounds.center.y, new_center.z-new_group.bounds.center.z) 
        t = Geom::Transformation.translation(transVec)
        new_group.transform!(t)###then move
        ### ending dialog...
        @msg = "提醒: 是否删除源对象 ?"
    	Sketchup::set_status_text(@msg)
		### v3.7
		### we explode the copy always
		xents=[]; xents=new_group.explode if new_group.valid?
		nents=[]; xents.each{|e|nents << e if e.is_a?(Sketchup::Drawingelement)}
		@faces=[]
	    ents.each{|e|@faces << e if e.valid? && e.class==Sketchup::Face}
	    self.gluer(nents)
		@ss.clear
		nents.each{|e|@ss.add(e) if e.valid?}
		### v3.7
    	if UI.messagebox(" 是否删除源对象 ?    ",MB_YESNO,"")==6 ### 6=YES 7=NO
            copy_group.erase! if copy_group.valid?        
      	else ### NO
            copy_group.explode if copy_group.valid?
	        @faces=[]
	        ents.each{|e|@faces << e if e.valid? && e.class==Sketchup::Face}
			self.gluer(sents)
			@ss.clear
            sents.each{|e|@ss.add(e) if e.valid?}
    	end#if
		###
		@ss.add(@locked) if @locked[0]
		###
       @model.commit_operation###v3.7
    	###
        self.reset()
        Sketchup.send_action("selectSelectionTool:")
    end#def
	
	def gluer(gents) ### v3.7
	  return nil unless @is2ds[0]
	  return nil unless @faces[0]
	  gents.each{|instance|
	    next unless instance.valid?
	    next unless instance.class==Sketchup::ComponentInstance
		next unless @is2ds.include?(instance.definition)
	    tr=instance.transformation
  	    co=tr.origin
	    cz=tr.zaxis
	    @faces.to_a.each{|face|
          next unless face.valid?
		  next unless face.parent==instance.parent
          if face.classify_point(co)==Sketchup::Face::PointInside && face.normal.parallel?(cz)
	        instance.glued_to=face
            break
	      end#if
	    }
	  }
	end### gluer v3.7
    
    def mirror_point(new_p)
        if @transformationType == @MIRROR_AT_PLANE
            if !new_p.on_plane?(@plane)
                mirrorp = new_p.project_to_plane(@plane)
                @trans = Geom::Transformation.scaling(mirrorp, -1.0)
                new_p.transform!(@trans)
            end
        elsif @transformationType == @MIRROR_AT_LINE
            if !new_p.on_line?(@line)
                mirrorp = new_p.project_to_line(@line)
                @trans = Geom::Transformation.scaling(mirrorp, -1.0)
                new_p.transform!(@trans)
            end
        elsif @transformationType == @MIRROR_AT_POINT
            if !(new_p == @point)
                new_p.transform!(@trans)
            end
        end
    end#def
    
end # class MirrorTool
### menus ##############################################################
unless file_loaded?(File.basename(__FILE__))###
  # UI.menu("Plugins").add_item("3点镜像"){Sketchup.active_model.select_tool(MirrorTool.new())}
### Context menu
#=begin
   UI.add_context_menu_handler do |menu|### v3.5 TIG
      if Sketchup.active_model.selection[0]
        #menu.add_separator
        menu.add_item("TC-3点镜像"){Sketchup.active_model.select_tool(MirrorTool.new())}
      end#if
   end#do menu###
#=end
### Toolbar - only adds the toolbar IF there's an icon png with it ###v3.2
  png=File.join(File.dirname(__FILE__),"MI.png")
  if FileTest.exist?(png)
   tb=UI::Toolbar.new("Mirror")
   cmd=UI::Command.new("MI"){Sketchup.active_model.select_tool(MirrorTool.new())}
   cmd.large_icon=png
   cmd.small_icon=png
   cmd.tooltip="3点镜像"
   cmd.status_bar_text="将选中的物体按对称点、对称轴或者对称面，进行镜像；可以选择是否保留源对象。"
   tb.add_item(cmd)
   if tb.get_last_state==TB_NEVER_SHOWN
      tb.show
   elsif tb.get_last_state==TB_VISIBLE
      tb.restore
   end#if state
 end#if toolbar
   ###
end#if loaded
file_loaded(File.basename(__FILE__))
###
