﻿module Ladb
  module OpenCutList


  
  ####################################################### RENDU 3D APPLIQUE LES MATERIAUX SUR LE FACES A L'IMPORTATION DU MOBILIER ###################################################
############################################################
$uStrings = LanguageHandler.new("utilities.strings")

def self.create_face_from_selection
    ss = Sketchup.active_model.selection
    
    # Get an Array of all of the selected Edges
    edges = ss.find_all { |e| e.kind_of?(Sketchup::Edge) }
    
    # We need at least 3 Edges
    if( edges.length < 3 )
        UI.messagebox($uStrings.GetString("You must select at least three Edges"))
        return nil
    end
    
    # Try to create a Face from the Edges in the active component
    face = nil
    begin
        face = Sketchup.active_model.active_entities.add_face edges
    rescue
    end
    
    # If a Face wasn't created, then there should be an error message
    # telling what the problem was
    if( not face )
        if( $! )
            msg = $!.message
        else
            msg = $uStrings.GetString("Create Face from Edges failed")
        end
        UI.messagebox msg
    end
    
    # Return the Face that was created
    face
end

#-------------------------------------------------------------------------
# This example show how you can write a simple tool in Ruby.  To implement
# a tool, you have to implement the methods that you want to handle.
# In this example the following methods are implementd:
# onMouseMove is called when a mouse move event is received.
# draw is handled to draw the tool

# This tool show coordinates of points and screen positions.
# It also shows lengths of selected Edges or areas of selected Faces.

# To see this tool in action, type the following in the Ruby console window:
# Sketchup.active_model.select_tool TrackMouseTool.new

class TrackMouseTool

# The activate method is called when a tool is first activated.  It is not
# required, but it is a good place to initialize stuff.
def activate
    @ip = Sketchup::InputPoint.new
    @iptemp = Sketchup::InputPoint.new
    @displayed = false
end

# onMouseMove is called whenever SketchUp gets a mouse move event.  It is called
# a lot, so you should try to make it as efficient as possible.
def onMouseMove(flags, x, y, view)
    # show the screen position in the VCB
    Sketchup::set_status_text("#{x}, #{y}", SB_VCB_VALUE)
    
    # get a position in the model and show it in a tooltip
    @iptemp.pick view, x, y
    if( @iptemp.valid? )
        changed = @iptemp != @ip
        @ip.copy! @iptemp
        pos = @ip.position;
        
        # get the text for the position
        msg = @ip.tooltip
        if( msg.length > 0 )
            msg << " "
        end
        #msg << pos.to_s
        msg << "( #{Sketchup.format_length(pos.x)},#{Sketchup.format_length(pos.y)},#{Sketchup.format_length(pos.z)} )"

        
        # See if it is on any special geometry
        if( @ip.vertex == nil )
            if( @ip.edge )
                if( @ip.depth > 0 )
                    length = @ip.edge.length(@ip.transformation)
                else
                    length = @ip.edge.length
                end
                msg <<  "\n       " + $uStrings.GetString("length") + " = #{Sketchup.format_length(length)}"
            elsif( @ip.face )
                if( @ip.depth > 0 )
                    area = @ip.face.area(@ip.transformation)
                else
                    area = @ip.face.area
                end
                msg << "\n       " + $uStrings.GetString("area") + " = #{Sketchup.format_area(area)}"
            end
        end

        msg2 = msg.gsub(/\n/,' ')
        Sketchup::set_status_text msg2
        
        # set the tooltip to show this message
        view.tooltip = msg
        
        # see if we need to update the display for this point
        if( changed and (@ip.display? or @displayed) )
            # This tells the view that we want it to update itself
            view.invalidate
        end
    end
    
end

# onLButtonDown is called when the user presses the left mouse button
def onLButtonDown(flags, x, y, view)
    Sketchup::set_status_text $uStrings.GetString("Left button down at") + " (#{x}, #{y})"
end

# onLButtonUp is called when the user releases the left mouse button
def onLButtonUp(flags, x, y, view)
    Sketchup::set_status_text $uStrings.GetString("Left button up at") + " (#{x}, #{y})"
end

# draw is optional.  It is called on the active tool whenever SketchUp
# needs to update the screen.
# in this case, we display an input point if needed
def draw(view)
    if( @ip.valid? && @ip.display? )
        @ip.draw view
        @displayed = true
    else
        @displayed = false
    end
end

def getInstructorContentDirectory
    "Query"
end

end

#-----------------------------------------------------------------------------
# Make selected components unique.
# This looks at all selected components.
def self.make_selected_components_unique

    # First get all selected components
    ss = Sketchup.active_model.selection
    components = ss.find_all {|e| e.kind_of?(Sketchup::ComponentInstance) }
    
    # Create a Hash.  The keys are the guids of all of the component definitions
    # for selected components.  The values are arrays of all of the component instances
    # that have that definition
    definitions = {}
    for component in components
        key = component.definition.guid
        a = definitions[key]
        if( a )
            a.push component
        else
            definitions[key] = [component]
        end
    end
    
    # Now make each collection of component instances have a new unique definition
    definitions.each_value do |a|
        # a is an array of component instances that all share the same definition
        # We will make the first one have a new unique definition, and all of the
        # other ones use that new definition
        definition = nil
        for component in a
            if( definition )
                # We already have a new definition.  Make this component use it
                component.definition = definition
            else
                # Make the component have a unique definition and save it
                component.make_unique
                definition = component.definition
            end
        end
    end

    true
end

#----------------------------------------------------------------------------
# Add things to the Utilities menu
if( not $utilities_menu_loaded )
   # add_separator_to_menu("Tools")
   # utilities_menu = UI.menu("Tools").add_submenu($uStrings.GetString("Utilities"))

    #utilities_menu.add_item($uStrings.GetString("Create Face")) { create_face_from_selection }
   # utilities_menu.add_item($uStrings.GetString("Query Tool")) { Sketchup.active_model.select_tool TrackMouseTool.new }
    #utilities_menu.add_item($uStrings.GetString("Fix Non-planar Faces")) { Sketchup.send_action "fixNonPlanarFaces:" }

  #  $utilities_menu_loaded = true
end



##################################################
	class Select_Through_Tool
		def enableVCB?
			if @state == "MOVE2"
				return true
			end
		end
		def onUserText(text, view)
			if @state == "MOVE2"
				begin
					value = text.to_l
				rescue
					# Error parsing the text
					UI.messagebox("无法将 #{text} 转换为长度")
					value = nil
					Sketchup::set_status_text "", SB_VCB_VALUE
				end
				return if ! value

				vec = @ip2.position - @mpt1
				if( vec.length == 0.0 )
					UI..messagebox("零长度无效!")
					return
				end
				vec.length = value
				dx = vec.x
				dy = vec.y
				dz = vec.z
				do_stretch(dx,dy,dz)
				reset(@rectmode)
			end
        end
		def initialize()
			@option_comp = "MAKE UNIQUE"#"NOT STRETCH"#"MODIFY ALL"
			@option_curve = true
			@option_arc = true
		end
		def reset(rectmode=true)
			@dt = Time.now.to_f#右クリックの時間を図ることでドラッグか点指定かを見分ける
			@ldn = false#画面範囲外で右クリックを話した場合の対策用
			@pts = []
			
			@spts = []
			@movelist = Hash.new
			@drawn = false
			@rectmode= rectmode
			@state="DRAW AREA"

			#@state="MOVE"
			@mpt1 = []
			@mpt2 = []
            @ip.clear
            @ip1.clear
            @ip2.clear
			@undobuff = 0
			
			@modify_dflist = []
			Sketchup.active_model.active_view.lock_inference
			Sketchup.active_model.selection.clear
		end
		def activate
            @ip  = Sketchup::InputPoint.new
            @ip1 = Sketchup::InputPoint.new
            @ip2 = Sketchup::InputPoint.new
			Sketchup.active_model.active_view.invalidate
			reset
		end
		def deactivate(view)
			#p "deactive"
			if @undobuff > 0
				Sketchup.undo
			end
			view.invalidate if @drawn == false
		end
		def onCancel(reason, view)
			#p "cancel"
			if @undobuff > 0
				Sketchup.undo
			end
			reset(@rectmode)
		end
		def getMenu(menu)
			if @state=="DRAW AREA"
			#elsif @state=="MOVE"
			#end
				if @rectmode and @rectmode == true
					menu.add_item("切换模式 (点击进行多点框选)") {
						reset(false)
					}
				else
					menu.add_item("切换模式 (拖动进行矩形框选)") {
						reset(true)
					}
					if @pts.size > 0
						menu.add_item("回退至上一点") {
							@pts.pop
						}
						menu.add_item("清除所有点") {
							@pts = []
						}
					end
					menu.add_item("关闭") {
						view = Sketchup.active_model.active_view
						@state="MOVE1"
						do_select_through()
						#do_select_through(view)
						#do_stretch
						@pts = []
					}
				end
			else
				menu.add_item("按数字输入") {
					prompts = ["X","Y","Z"]
					defaults = ["0","0","0"]
					res = UI.inputbox( prompts , defaults , "拉伸值" )
					return if not res
					dx = res[0].to_l
					dy = res[1].to_l
					dz = res[2].to_l
					do_stretch(dx,dy,dz)
					reset(@rectmode)
				}
			end
		end

		def onReturn(view)
			if @state=="DRAW AREA"
			elsif @state=="MOVE2"
				dvec=@mpt1.vector_to(@mpt2)
				dx = dvec.x
				dy = dvec.y
				dz = dvec.z
				do_stretch(dx,dy,dz)
			end
		end
		def onLButtonDown(flags, x, y, view)
			if @state=="DRAW AREA"
				if @rectmode == true
					if @ldn == false
						@dt = Time.now.to_f
						@pts[0] = [x,y,0]
					end
					@ldn = true
					#view.invalidate
				end

			elsif @state=="MOVE2"

			end
		end
		def onLButtonUp(flags, x, y, view)
			if @state=="DRAW AREA"
				if @rectmode == true

					@pts[1] = [x,y,0]
					@ldn = false
					@state="MOVE1"
					do_select_through()

				else
					@pts.push [x,y,0]
				end
			elsif @state=="MOVE1"
                if( @ip1.valid? )
                    @mpt1=@ip1.position
					@state="MOVE2"
                end
			elsif @state=="MOVE2"
                if( @ip2.valid? )
					@mpt2=@ip2.position
					dvec=@mpt1.vector_to(@mpt2)
					dx = dvec.x
					dy = dvec.y
					dz = dvec.z
					do_stretch(dx,dy,dz)
                end
			end
		end
		def onMouseEnter(view)

		end
		def onMouseLeave(view)

		end
		def onMouseMove(flags, x, y, view)
			if @state=="MOVE1"
				Sketchup::set_status_text "拖动框选的第一点"
				@ip.pick view, x, y
				view.invalidate if( @ip.display? )
				@ip1.copy! @ip
				view.tooltip = @ip1.tooltip
				view.invalidate
			elsif @state=="MOVE2"
                # Getting the next end
                Sketchup::set_status_text "拖动框选的第二点"
                Sketchup::set_status_text "Length ", SB_VCB_LABEL
                @ip2.pick view, x, y, @ip1
                if( @ip2.valid? )
                    length = @ip1.position.distance(@ip2.position)
                    Sketchup::set_status_text length.to_s, SB_VCB_VALUE
                end
                view.tooltip = @ip2.tooltip if( @ip2.valid? )
                view.invalidate
			else
				if @rectmode==true
					Sketchup::set_status_text "按住鼠标左键拖动以选择需要移动的部分的点，或者右键切换为点击划区选择"
					if @ldn == true
						@pts[1] = [x,y,0]
						view.invalidate
					end
				else
					Sketchup::set_status_text "通过点击进行多边形区域框选"
					@tpt = [x,y,0]
					view.invalidate
				end
			end
		end
		def resume(view)

		end
		def suspend(view)

		end
		def draw(view)
			if @pts != []
				vpts = []
				if @rectmode == true and @pts.size > 1
					vpts[0] = [@pts[0][0], @pts[0][1],0]
					vpts[1] = [@pts[0][0], @pts[1][1],0]
					vpts[2] = [@pts[1][0], @pts[1][1],0]
					vpts[3] = [@pts[1][0], @pts[0][1],0]
				else @pts.size > 0
					@pts.each{|pt|
						vpts.push pt
					}
					vpts.push @tpt if @tpt
				end
				if vpts.size > 1
					paintcolor = Sketchup::Color.new(255, 0, 0)
					paintcolor.alpha = 128
					view.line_width=2
					view.drawing_color = paintcolor
					view.draw2d(GL_LINE_LOOP,vpts)
				end
			end
			#view.invalidate
			if @spts != []
				view.draw_points( @spts, 5, 2, "red" )
			end

			if not @state == "DRAW AREA"
			if @mpt1 != [] and @ip2.valid? 
				view.set_color_from_line(@mpt1, @ip2.position)
				if view.inference_locked?
					view.line_width=2
				end
				view.draw_line( @mpt1 , @ip2.position )
			end
			if @mpt1 != []
				view.draw_points @mpt1, 8, 2, "green"
			elsif@ip1.valid? 
				view.draw_points @ip1.position, 8, 2, "green"
			end
			if @ip2.valid? 
				view.draw_points @ip2.position, 8, 2, "green"
			end
			end
			@drawn = true

		end
        def onKeyDown(key, rpt, flags, view)
            if( key == CONSTRAIN_MODIFIER_KEY )
                if( view.inference_locked? )
					#拘束を解除
                    view.lock_inference
                elsif( @state == "MOVE2" )
                    view.lock_inference @ip2 , @ip1
                end
                view.invalidate
            end
        end

        def onKeyUp(key, rpt, flags, view)
            if( key == CONSTRAIN_MODIFIER_KEY && view.inference_locked? )
				#拘束を解除
                view.lock_inference
            end
            view.invalidate
        end
		def do_stretch(dx,dy,dz)
			opt = [0,0,0]
			dpt = [dx,dy,dz]
			#mtr = Geom::Transformation.translation(movevec)
			Sketchup::set_status_text "拉伸实体: 请稍候"
			Sketchup.active_model.start_operation "Stretch Move",true
				pcount = 0
				pmax = @movelist.keys.size
				@movelist.keys.each{|parent|
					getarr = @movelist[parent]
					if getarr
						submovelist = getarr[0]
						gtr = getarr[1]
						opt2 = opt.transform( gtr.inverse )
						dpt2 = dpt.transform( gtr.inverse )
						movevec = opt2.vector_to(dpt2)
						mtr = Geom::Transformation.translation(movevec)
						vtlist , submovelist = submovelist.partition{|e| e.kind_of? Sketchup::Vertex }
						crvlist , submovelist = submovelist.partition{|e| e.kind_of? Array }
						parent.entities.transform_entities( mtr , submovelist )
						parent.entities.transform_entities( mtr , vtlist )
						if crvlist != []
							crvlist.each{|crv,vts|
								p crv
								p vts
								mpts = []
								crv.vertices.each{|vt|
									if vts.index( vt )
										mpts.push vt.position.transform( mtr )
									else
										mpts.push vt.position
									end
								}
								crv.move_vertices( mpts )
							}
						end
					end
					pcount += 1
					Sketchup::set_status_text "Stretching Entities:Please wait...#{pcount}/#{pmax}" if ( pcount % 50 )== 0
				}
			Sketchup.active_model.commit_operation
			reset(@rectmode)
		end

		def get_screen_point(x,y,view)
			camera = view.camera
			cdir = camera.eye.vector_to(camera.target)
			bbs = Sketchup.active_model.bounds
			dist = bbs.center.distance(camera.eye) + bbs.diagonal
			cdir.length = dist
			pos = camera.eye + cdir
			plane = [ pos, camera.direction ]
			line = view.pickray( x, y )
			ipt = Geom.intersect_line_plane(line, plane)
			return ipt
		end
		def do_select_through()
			Sketchup::set_status_text "获取拉伸的实体: 请稍候..."
			
			view = Sketchup.active_model.active_view
			pts = []
			if @rectmode==true
				pts[0] = [ @pts[0][0], @pts[0][1] , 0 ]
				pts[1] = [ @pts[0][0], @pts[1][1] , 0 ]
				pts[2] = [ @pts[1][0], @pts[1][1] , 0 ]
				pts[3] = [ @pts[1][0], @pts[0][1] , 0 ]
			else
				pts = @pts
			end
			if pts.size < 4
				reset(@rectmode)
				return
			end
			areaface = pts
			sels = Sketchup.active_model.selection
			@count = 0
			@undobuff = 0
			Sketchup.active_model.start_operation "Stretch Move",true,true
			select_through_in_area( sels , areaface )
			Sketchup.active_model.commit_operation# if @undobuff > 0
			
			#@movelist.values.each{|sumlist,gtr|
			#	p sumlist
			#}
			#@spts = []
			hashpts = Hash.new
			vtscount = 0
			grpcount = 0
			crvcount = 0
			othercount = 0
			@movelist.keys.each{|parent|
				getarr = @movelist[parent]
				if getarr
					submovelist = getarr[0]
					gtr = getarr[1]
					submovelist.each{|e|
						if e.kind_of? Sketchup::Vertex
							vtscount += 1
							pt3 = e.position.transform( gtr )
							hashpts[pt3] = 0
							Sketchup.set_status_text "Selected::#{vtscount}Vertices and #{grpcount}Components/Groups and #{crvcount}Curves and #{othercount}Others." if( vtscount % 50 ) == 0
						elsif e.kind_of? Sketchup::Group or e.kind_of? Sketchup::ComponentInstance
							grpcount += 1
							Sketchup.set_status_text "Selected::#{vtscount}Vertices and #{grpcount}Components/Groups and #{crvcount}Curves and #{othercount}Others." if( grpcount % 10 ) == 0
						elsif e.kind_of? Array
							e.each{|crv,vts|
								if vts
									crvcount += 1
									if vts.kind_of? Array
										vts.each{|vt|
											pt3 = vt.position.transform(gtr)
											hashpts[pt3] = 0
										}
									elsif vts.kind_of? Sketchup::Vertex
										pt3 = vts.position.transform(gtr)
										hashpts[pt3] = 0
									end
								end
							}
							Sketchup.set_status_text "Selected::#{vtscount}Vertices and #{grpcount}Components/Groups and #{crvcount}Curves and #{othercount}Others." if( grpcount % 5 ) == 0
						else
							othercount += 1
							Sketchup.set_status_text "Selected::#{vtscount}Vertices and #{grpcount}Components/Groups and #{crvcount}Curves and #{othercount}Others." if( othercount % 5 ) == 0
						end
					}
				end
			}
			#ArrayよりもHashの方が高速なので一度ハッシュで受けてからKeysのArrayを取り出す
			@spts = hashpts.keys
			Sketchup.set_status_text "Selected::#{vtscount}Vertices and #{grpcount}Components/Groups and #{crvcount}Curves and #{othercount}Others."
			@pts = []
			if vtscount == 0 and grpcount == 0 and crvcount == 0 and othercount == 0
				reset(@rectmode) 
			end
			view.invalidate
			#view.refresh
			#何も選択が無いときは最初に戻る

		end
		def select_through_in_area( sels , areaface , ents = Sketchup.active_model.active_entities , gtr = Geom::Transformation.new )
			crvlist = []
			vts_incurve = Hash.new#カーブ内の頂点で重複を避けるためのストッカー
			parent = ents.parent
			getarr = @movelist[ parent ]
			if getarr and getarr != []
				submovelist , tr2 = getarr
			else
				submovelist = []
			end
			ents.each{|e|
				if e.visible? and e.layer.visible?
					if submovelist.index(e)
						#既に選択されている場合
					else
						#選択されていない場合
						if e.kind_of? Sketchup::Group and e.locked? == false
							chk = grp_is_in_area( areaface , e , gtr ) 
							if chk == 8
								sels.add e
								submovelist.push e
								
								#このオブジェクト内の選択は除外する処理
							elsif chk == -9
							else
								if area2d_from_bounds( areaface , e , gtr )==true
									if e.entities.find{|e2| e2 != nil }.parent.count_instances > 1
										@undobuff += 1
										#この処理を煮詰める必要がある このグループ内の頂点などが選択されない場合は飛ばすように。
										
										#Sketchup.active_model.start_operation "Stretch Move",true,true
											e.make_unique
										#Sketchup.active_model.commit_operation
									end
									select_through_in_area( sels , areaface , e.entities , gtr * e.transformation )
								end
							end
						elsif e.kind_of? Sketchup::ComponentInstance and e.locked? == false
							chk = grp_is_in_area( areaface , e , gtr ) 
							if chk == 8
								sels.add e
								submovelist.push e
								#このオブジェクト内の選択は除外する処理
							elsif chk == -9
							else
								if @option_comp != "NOT STRETCH"
									if area2d_from_bounds( areaface , e , gtr )==true
										if !@modify_dflist.index( e.definition )
											if e.definition.count_instances > 1
												@undobuff += 1
												#Sketchup.active_model.start_operation "Stretch Move",true,true
												#この処理を煮詰める必要がある このグループ内の頂点などが選択されない場合は飛ばすように。
													e.make_unique
												#Sketchup.active_model.commit_operation
											end
											select_through_in_area( sels , areaface , e.definition.entities , gtr * e.transformation )
											if @option_comp == "MODIFY ALL"
												@modify_dflist.push e.definition
											end
										end
									end
								end
							end
						elsif e.kind_of? Sketchup::Edge
						#エッジの場合はさらに複雑な処理をする
							crv = e.curve
							if crv
								if !crvlist.index(crv)
									crvlist.push crv
									if curve_is_in_area( areaface, crv , gtr , vts_incurve )
										sels.add crv.edges
										submovelist.push crv
									end
								end
							else
								if edge_is_in_area( areaface, e , gtr )
									#sels.add e
									#submovelist.push e
									
									#このエッジ内の選択点は除外する処理
								end
							end
						elsif e.kind_of? Sketchup::ConstructionLine
							if cline_is_in_area( areaface, e , gtr )
								sels.add e
								submovelist.push e
							end
						elsif e.kind_of? Sketchup::ConstructionPoint
							if cpoint_is_in_area( areaface, e , gtr )
								sels.add e
								
								submovelist.push e
							end
						end
					end
				end
				@count += 1
				Sketchup.set_status_text "请稍后...#{@count}" if ( @count % 500 ) == 0
			}
			#moves = [ submovelist , gtr ]
			getarr = @movelist[ parent ]
			if getarr and getarr != []
				submovelist2 , tr2 = getarr

			else
				submovelist2 = []
			end
			submovelist2.concat(submovelist)
			
			@movelist[ parent ]= [ submovelist2 , gtr ]

			#####ここでプレビュー用の点座標群を更新する
			##
			
			#submovelist2.each{|e|
			#	if e.kind_of? Sketchup::Vertex
			#		pt3 = e.position.transform( gtr )
			#		if not @spts.index( pt3 )
			#			@spts.push pt3
			#		end
			#	end
			#}
			@drawn = false
		end
		def grp_is_in_area( areaface , ent , gtr )
			bbs = ent.bounds
			view = Sketchup.active_model.active_view
			#eye = view.camera.eye
			chk = 0
			bbareaface = []
			(0..7).each{|i|
				pt = bbs.corner(i).transform(gtr)
				bbareaface.push view.screen_coords( pt )
				if check_pt_in_area( areaface , bbs.corner(i) , gtr ) == true
					chk += 1
				else
					chk += -1
				end
			}
			if chk == -8
				bbchk = 0
				areaface.each{|vpt|
					if check_pt_in_area( bbareaface ,nil , gtr ,vpt) == true
						bbchk += 1
					else
						bbchk += -1
					end
				}
				if bbchk == -8
					chk = -9
				end
			end
			return chk
		end

		def cpoint_is_in_area( areaface , cpoint , gtr )
			pts = []
			pts[0] = cpoint.position
			chk = check_pts_in_area( areaface , pts , gtr )
			return chk
		end
		def cline_is_in_area( areaface , cline , gtr )
			pts = []
			pts[0] = cline.start
			pts[1] = cline.end
			return false if not pts[0] or not pts[1]
			chk = check_pts_in_area( areaface , pts , gtr )
			return chk
		end
		def curve_is_in_area( areaface , crv , gtr , vts_incurve )
			movelist = []
			parent = crv.parent
			getarr = @movelist[ parent ]
			if getarr and getarr != []
				submovelist , tr2 = getarr
			else
				submovelist = []
			end
			submovelist = [] if submovelist == nil
			
			chk = 0
			cnt = 0
			#vts = []
			#vts[0] = crv.vertices[0]
			#vts[1] = crv.vertices[-1]
			vts = crv.vertices
			vts.each{|vt|
				if check_pt_in_area( areaface , vt.position ,gtr )
					movelist.push vt
					chk += 1
				else
					chk += -1
				end
				cnt += 1
			}
			#if chk == cnt
				#movelist.each{|mvt|
				#	submovelist.reject!{|k,v| k == mvt }
				#}
				#@movelist[parent] = [ submovelist , gtr ]
			#	return true

			if chk == cnt
				return true
			elsif chk != -cnt# and !crv.kind_of? Sketchup::ArcCurve
				if crv.kind_of? Sketchup::ArcCurve
					#円弧カーブで途中の点が選択される場合は一度通常のカーブにする
					#この際idは変化しないことを確認した
					pts = crv.vertices.map{|vt| vt.position }
					crv.edges[0].explode_curve
					eds = parent.entities.add_curve( pts )
					crv = eds[0].curve
				end

				if @option_curve == true
					crv_vts = [nil]
					movelist.each{|mvt|
						#if not submovelist.index(mvt)
						if !vts_incurve.key?(mvt)
							vts_incurve[mvt] = 0
							crv_vts.push mvt
						end
						#end
					}
					crarr = [ crv , crv_vts ]
					submovelist.push crarr
					@movelist[parent] = [ submovelist , gtr ]
					return nil
				end
			end
			return nil
		end
		def edge_is_in_area( areaface , ed , gtr )
			
			parent = ed.parent
			getarr = @movelist[ parent ]
			
			if getarr and getarr != []
				submovelist , tr2 = getarr
			else
				submovelist = []
			end
			submovelist = [] if submovelist == nil
			
			movelist = []
			chk = 0
			cnt = 0
			ed.vertices.each{|vt|
				#頂点がカーブ内にある場合は除外する
				#curve_interiorでは上手く判定できない場合がある
				#if vt.curve_interior? == nil
				if vt.edges.find{|ed| ed.curve } == nil
					#p vt.curve_interior?
					if check_pt_in_area( areaface , vt.position ,gtr )
						movelist.push vt
						chk += 1
					else
						chk += -1
					end
					cnt += 1
				else
				#elsif !vts_incurve.key?(vt)
				#	vts_incurve[vt] = 0
				#	vts = [];vts.push vt
				#	submovelist.push [ vt.curve , vts ]
				#	@movelist[parent] = [ submovelist , gtr ]
				end
			}

			movelist.each{|mvt|
				if not submovelist.index(mvt)
					submovelist.push mvt
				end
			}
			@movelist[parent] = [ submovelist , gtr ]
			#if chk == cnt
				#movelist.each{|mvt|
				#	submovelist.reject!{|k,v| k == mvt }
				#}
				#@movelist[parent] = [ submovelist , gtr ]
			#	return true
			#else

			return nil
			#end
			#pts = ed.vertices.map{|vt| vt.position }
			#chk = check_pts_in_area( areaface , pts , gtr )
			#return chk
		end
		def check_pts_in_area( areaface , pts , gtr )
			chk = 0
			cnt = 0

			pts.each{|pt|
				if check_pt_in_area( areaface , pt ,gtr )
					chk += 1
				else
					chk += -1
				end
				cnt += 1
			}

			if chk == cnt
				return true
			else
				return nil
			end
		end

		def check_pt_in_area( areaface , pt , gtr , xy = nil)
			#areafaceを画面上の座標として考える
			#geomにpoint_in_polygon_2Dがあったのでそれに乗り換え
			view = Sketchup.active_model.active_view
			#eye = Sketchup.active_model.active_view.camera.eye
			if !xy
				cpt = pt.transform(gtr)
				xy = view.screen_coords( cpt )
			end
			xy.z = 0.0
			if Geom.point_in_polygon_2D(xy, areaface,true) == true
				return true
			end
			return false
=begin
	#偏角の合計から内外を判定する
			sumang = 0.0
			indice = []
			(areaface.size).times{|i|
			indice.push( i -1 )
			}
			
			indice.each{|i|
				apt = areaface[i]
				bpt = areaface[i+1]
				vec1 = xy.vector_to(apt)
				vec2 = xy.vector_to(bpt)
				
				ang1 = Math.atan2( vec1.y , vec1.x ).radians
				ang2 = Math.atan2( vec2.y , vec2.x ).radians
				ang1 = 360 + ang1 if ang1 < 0.0
				ang2 = 360 + ang2 if ang2 < 0.0
				ang3 = ang2- ang1
				if ang3 > 180
					ang3 = ang3 - 360
				elsif ang3 < -180
					ang3 = ang3 + 360
				end
				sumang += ang3
			}

			if sumang.abs < 0.01
				#偏角の合計が0に近いので外
				return false
			else
				#中
				return true
			end
=end
		end
		def area2d_from_bounds( areaface , e , gtr )
			
			#バウンディングボックスの頂点で囲まれた範囲内にareaface点群が含まれるかをチェックする
			gtr2 = gtr * e.transformation
			view = Sketchup.active_model.active_view
			cdir = view.camera.direction
			cdir2 = cdir.transform( gtr2.inverse )
			if e.kind_of? Sketchup::Group
				bbsc = e.local_bounds
			else
				bbsc = e.definition.bounds
			end
			bareas = []
			if cdir.x < 0
				bareas.push [1,3,7,5].map{|i| view.screen_coords( bbsc.corner(i).transform(gtr2) ) }
			elsif cdir.x > 0
				bareas.push [0,2,6,4].map{|i| view.screen_coords( bbsc.corner(i).transform(gtr2) ) }
			end
			if cdir.y < 0
				bareas.push [2,3,7,6].map{|i| view.screen_coords( bbsc.corner(i).transform(gtr2) ) }
			elsif cdir.y > 0
				bareas.push [0,1,5,4].map{|i| view.screen_coords( bbsc.corner(i).transform(gtr2) ) }
			end
			if cdir.z < 0
				bareas.push [4,5,7,6].map{|i| view.screen_coords( bbsc.corner(i).transform(gtr2) ) }
			elsif cdir.z > 0
				bareas.push [0,1,3,2].map{|i| view.screen_coords( bbsc.corner(i).transform(gtr2) ) }
			end

			bareas.each{|barea|
				#Sketchup.active_model.active_entities.add_face( barea )
				areaface.each{|xy|
					if check_pt_in_area( barea , nil , gtr , xy) == true
						#p "check_pt_in_area true"
						return true
					end
				}
				
			}
			#全ての頂点はbareas外にある、線分の交差判定を行う
			bareas.each{|barea|
				if check_int_bounds_area( areaface , barea ) == true
					#p "check_int_bounds_area true"
					return true
				end
			}
			return false
		end
		def check_int_bounds_area( areaface , barea )
			p "check_int_bounds_area"
			[[0,1],[1,2],[2,3],[3,0]].each{|i1,i2|
				xy = [ barea[i1] , barea[i2] ]
				if check_int_line_area( areaface , xy )
					return true
				else
				end
			}
			return false
		end
		def check_int_line_area( areaface , xy1 )
			view = Sketchup.active_model.active_view
			xy1[0].z = 0.0
			xy1[1].z = 0.0
			indice = []
			(areaface.size).times{|i|
			indice.push( i -1 )
			}
			xy2 = []
			indice.each{|i|
				xy2[0] = areaface[i]
				xy2[1] = areaface[i+1]
				xy2[0].z = 0.0
				xy2[1].z = 0.0
				if check_intersect_edges(xy1,xy2) == true
					return true
				end
			}
			
			return false
		end
		def check_intersect_edges(pts1,pts2)
			pts1[0].z = 0
			pts1[1].z = 0
			pts2[0].z = 0
			pts2[1].z = 0
			
			line1 = [pts1[0],pts1[0].vector_to(pts1[1])]
			dist1 = pts1[0].distance(pts1[1]) + 0.0.mm
			line2 = [pts2[0],pts2[0].vector_to(pts2[1])]
			dist2 = pts2[0].distance(pts2[1]) + 0.0.mm
			if dist1 <= 0.0 or dist2 <= 0.0
			else
				ipt = Geom.intersect_line_line(line1, line2)
				if ipt
					ilen = ipt.distance(pts1[0]) + ipt.distance(pts1[1])
					ilen2 = ipt.distance(pts2[0]) + ipt.distance(pts2[1])
					if ilen <= ( dist1 *1.001 ) and ilen2 <= ( dist2 * 1.001 )
						return true
					end
				end
			end
			return false
		end
	end
	def self.add_menu()
	
	end


##########################################################################################################################################################################################
 
class MyEntitiesObserver < Sketchup::EntitiesObserver

TNT_COMPONENT_PATTERNS = []  
  PAINT_COMPONENT_PATTERNS = []
  DONT_PAINT_COMPONENT_PATTERNS = []
 
end

########################################################### CERATION DE LA CLASS SELF AVEC TOUTES LES METHODES DU PLUGIN AF-Custom 2 #############################################  

class A

class << self 
@@langue='ANG'

  path_to_components = File.join(File.dirname(__FILE__), 'Components')
 
	@@path_to_resources = File.join(File.dirname(__FILE__), 'Resources')
 

###################################################################################################################################################################################








#******************************画墙开始*******************************************
	
	
	
	### MENU & TOOLBARS ### --------------------------------------------------

	#unless file_loaded?( __FILE__ )      
	#	cmd_ExtrudeAlongPath = UI::Command.new('挤压矩形') { ExtrudeAlongPath.main }
	#	cmd_ExtrudeAlongPath.large_icon = 'Images/ExtrudeAlongPath_24.png'
	#	cmd_ExtrudeAlongPath.small_icon = 'Images/ExtrudeAlongPath_16.png'
	#	cmd_ExtrudeAlongPath.status_bar_text = '沿路径挤压矩形。'
	#	cmd_ExtrudeAlongPath.tooltip = '沿路径挤压矩形'

		# Menu
		#@menu.add_item( cmd_ExtrudeAlongPath )
		#@menu.add_separator

		# Toolbar
		#@toolbar.add_item( cmd_ExtrudeAlongPath )
		#@toolbar.add_separator
	#end # UI
	#file_loaded( __FILE__ )
  
  
	### METHODS ### ----------------------------------------------------------
	class ExtrudeAlongPath
		def self.main

		@error=0 ### 17/8/5 ###
		model = Sketchup.active_model
			
		begin
			model.start_operation("沿路径挤压矩形", true)
		rescue
			model.start_operation("沿路径挤压矩形")
		end

		entities = model.active_entities
		ss = model.selection

		if ss.empty?
		  UI.messagebox("未选择路径 !")
		  return nil
		end
		### #################################################################

		def self.get_vertices

		### this next bit is mainly thanks to Rick Wilson's weld.rb ###

			model=Sketchup.active_model
			ents=model.active_entities
			@sel=model.selection
			sl=@sel.length
			verts=[]
			edges=[]
			@newVerts=[]
			startEdge=startVert=nil

		#DELETE NON-EDGES, GET THE VERTICES

			@sel.each {|item| edges.push(item) if item.typename=="Edge"}
			edges.each {|edge| verts.push(edge.vertices)}
			verts.flatten!

		#FIND AN END VERTEX

			vertsShort=[]
			vertsLong=[]
			vertsEnds=[] ### 17/8/5 ### to ensure array is only ends ###
			verts.each do |v|
				if vertsLong.include?(v)
					vertsShort.push(v)
				else
					vertsLong.push(v)
				end
			end
			vertsLong.each do |v| ### 17/8/5 ### to ensure array is only ends ###
				if not vertsShort.include?(v)
					vertsEnds.push(v)
				end
			end ### 17/8/5 ### to ensure array is only ends ###
			if vertsEnds.length==0 ### i.e. it's looped ###
				### path start or end ? ### 17/8/5 ###
				if @theEnd==0
				  startVert=vertsLong.first
				  startEdge=startVert.edges.first
				else
				  startVert=vertsLong.last
				  startEdge=startVert.edges.first
				end
				###
				closed=true
			else
				if vertsEnds.length != 2
					UI.messagebox("所选路径有分叉或不连续 ! \n无法进行挤压 ! \n请重新选择单个连续路径... ") ### 17/8/5 ###
					@error=1
					return nil
				else
					### path start or end ? ### 17/8/5 ###
					if @theEnd==0
					  startVert=vertsEnds.first
					else
					  startVert=vertsEnds.last
					end
					###
					closed=false
					startEdge=startVert.edges.first
				end
			end
			@sel.clear


		#SORT VERTICES, LIMITING TO THOSE IN THE SELECTION SET

			if startVert==startEdge.start
				@newVerts=[startVert]
				counter=0
				while @newVerts.length < verts.length
					edges.each do |edge|
						if edge.end==@newVerts.last
							@newVerts.push(edge.start)
						elsif edge.start==@newVerts.last
							@newVerts.push(edge.end)
						end
					end
					counter+=1
					if counter > verts.length
						return nil if UI.messagebox("似乎出问题了，是否重试 ?", MB_YESNO)!=6
						@newVerts.reverse!
						reversed=true
					end
				end
			else
				@newVerts=[startVert]
				counter=0
				while @newVerts.length < verts.length
					edges.each do |edge|
						if edge.end==@newVerts.last
							@newVerts.push(edge.start)
						elsif edge.start==@newVerts.last
							@newVerts.push(edge.end)
						end
					end
					counter+=1
					if counter > verts.length
						return nil if UI.messagebox("似乎出问题了，是否重试 ?", MB_YESNO)!=6
						@newVerts.reverse!
						reversed=true
					end
				end
			end
			@newVerts.reverse! if reversed

		#CONVERT VERTICES TO POINT3Ds

			@newVerts.collect!{|x| x.position}

		### now have an array of vertices in order with NO forced closed loop ...

		end ### get_vertices

		### near or far ?
		@theEnd=0
		get_vertices
		if @error==1
		  return nil
		end

		### dialog #################################################################

		alignments = ["边缘","中心(底部)","中心","中心(侧边)"]
		enums = [alignments.join("|")]
		prompts = ["对齐: ","宽度: ","高度: "]

		if not @widthIn
		   values = ["边缘",100.mm,2700.mm]
		else
		   values = [@alignment,@widthIn,@heightInfirst]
		end

		results = inputbox prompts, values, enums, "参数设置"

		return nil if not results ### i.e. the user cancelled the operation

		@alignment,@widthIn,@heightInfirst = results

		@heightIn = @heightInfirst

		### #################################################################

		### #################################################################

		### restore selection set of edges and display them
		def self.edge_reselector (xnewVerts)
		 model = Sketchup.active_model
		 ents=model.active_entities
		 theEdgeX = []
		 0.upto(xnewVerts.length-2) do |i| ### 1/8/5
		   theEdgeX[i] = ents.add_line(xnewVerts[i],xnewVerts[i+1])  ### make vertices into edges
		 end
		 model.selection.clear
		 model.selection.add theEdgeX
		end
		###def


		### do main stuff #################################################################

		pt1 = @newVerts[0]
		pt2 = @newVerts[1]

		width = @widthIn
		if @widthIn == 0.mm ### can't be 0 ###
		   @widthIn = 100.mm
		   edge_reselector(@newVerts)
		   UI.messagebox("宽度不能为 0 !")
		   return nil
		end
		###
		if @heightIn == 0.mm ### can't be 0 ###
		   @heightIn = @widthIn
		end
		###
		if (pt1.x == pt2.x) and (pt1.y == pt2.y) ### vertical 1st path ###
		   vflag = 1
		   height = @heightIn
		else
		   vflag = 0
		   height = ((@heightIn * (pt1.distance pt2)) / (pt1.distance [pt2.x,pt2.y,pt1.z]))
		end
		###
		if @alignment == "边缘"
		   offL = width
		   offR = 0
		   heightUp = height
		   heightDn = 0
		   cpatt = "__" ### 17/8/5 ###
		end
		if @alignment == "中心(底部)"
		   offL = width / 2
		   offR = width / 2
		   heightUp = height
		   heightDn = 0
		   cpatt = "." ### 17/8/5 ###
		end
		if @alignment == "中心(侧边)"
		   offL = width
		   offR = 0
		   heightUp = height / 2
		   heightDn = height / 2
		   cpatt = "_" ### 17/8/5 ###
		end
		if @alignment == "中心"
		   offL = width / 2
		   offR = width / 2
		   heightUp = height / 2
		   heightDn = height / 2
		   cpatt = "-.-" ### 17/8/5 ###
		end
		### #################################################################

		if vflag == 1 ### vertical ### 7/8/5
		  vec = pt1.vector_to [(pt1.x + 1), pt1.y, pt1.z]
		else
		  vec = pt1.vector_to [pt2.x, pt2.y, pt1.z]
		end

		### #################################################################

		piBy2 = 1.5707963267948965 ### a radian right-angle for calculating vector offset to edges

		rotated_vecL = vec.transform(Geom::Transformation.rotation(pt1, [0,0,1], (0 + piBy2))) ### 7/8/5
		pt1_leftC = pt1.offset(rotated_vecL, offL)
		rotated_vecR = vec.transform(Geom::Transformation.rotation(pt1, [0,0,1], (0 - piBy2))) ### 7/8/5
		pt1_rightC = pt1.offset(rotated_vecR, offR)

		if vflag == 1 ### 1st path is vertical ### 7/8/5
		  pt1_left  = pt1_leftC.offset([-1,0,0], heightDn)
		  pt1_right = pt1_rightC.offset([-1,0,0], heightDn)
		  pt2_left  = pt1_leftC.offset([1,0,0], heightUp)
		  pt2_right = pt1_rightC.offset([1,0,0], heightUp)
		else
		  pt1_left  = pt1_leftC.offset([0,0,-1], heightDn)
		  pt1_right = pt1_rightC.offset([0,0,-1], heightDn)
		  pt2_left  = pt1_leftC.offset([0,0,1], heightUp)
		  pt2_right = pt1_rightC.offset([0,0,1], heightUp)
		end

		###

		ents=entities

		### #################################################################
		### add construction line along first vector ### 7/8/5
		cline1 = ents.add_cline(pt1,pt2)
		cline1.end=nil
		cline1.start=nil
		cline1.stipple=cpatt ### 17/8/5 ###


		### group the extrusion... ### 7/8/5 #################################################################

		group=entities.add_group
		entities=group.entities

		theFace = entities.add_face(pt1_right, pt1_left, pt2_left, pt2_right) ### 7/8/5

		if vflag == 1 and height > 0
		   theFace.reverse!
		end
		if vflag == 1 and width < 0
		   theFace.reverse!
		end
		if vflag == 1 and pt1.z > pt2.z
		   theFace.reverse!
		end

		### check start /end of path ###
		keypress = 6 ### = YES
		### ask if face starts at correct side ? ### 17/8/5 ###
		model.selection.clear
		model.selection.add [theFace]
		model.selection.add [theFace.edges]
		###
		if @alignment == "中心" || @alignment == "中心(底部)"
			keypress = 6
		else
			keypress = UI.messagebox("从这端开始 ?", MB_YESNO) ###
		end
		if keypress != 6
		  ### get rid of old stuff first ###
		  entities.erase_entities cline1
		  entities.erase_entities model.selection
		  edge_reselector(@newVerts)
		  @theEnd=1
		  get_vertices
		  ###
		  pt1 = @newVerts[0]
		  pt2 = @newVerts[1]
		  if (pt1.x == pt2.x) and (pt1.y == pt2.y) ### vertical 1st path ###
			 vflag = 1
			 height = @heightIn
		  else
			 vflag = 0
			 height = ((@heightIn * (pt1.distance pt2)) / (pt1.distance [pt2.x,pt2.y,pt1.z]))
		  end
		  if vflag == 1 ### vertical ### 7/8/5
			vec = pt1.vector_to [(pt1.x + 1), pt1.y, pt1.z]
		  else
			vec = pt1.vector_to [pt2.x, pt2.y, pt1.z]
		  end
		  rotated_vecL = vec.transform(Geom::Transformation.rotation(pt1, [0,0,1], (0 + piBy2))) ### 7/8/5
		  pt1_leftC = pt1.offset(rotated_vecL, offL)
		  rotated_vecR = vec.transform(Geom::Transformation.rotation(pt1, [0,0,1], (0 - piBy2))) ### 7/8/5
		  pt1_rightC = pt1.offset(rotated_vecR, offR)
		  if vflag == 1 ### 1st path is vertical ### 7/8/5
			pt1_left  = pt1_leftC.offset([-1,0,0], heightDn)
			pt1_right = pt1_rightC.offset([-1,0,0], heightDn)
			pt2_left  = pt1_leftC.offset([1,0,0], heightUp)
			pt2_right = pt1_rightC.offset([1,0,0], heightUp)
		  else
			pt1_left  = pt1_leftC.offset([0,0,-1], heightDn)
			pt1_right = pt1_rightC.offset([0,0,-1], heightDn)
			pt2_left  = pt1_leftC.offset([0,0,1], heightUp)
			pt2_right = pt1_rightC.offset([0,0,1], heightUp)
		  end
		  ### add NEW construction line along NEW first vector ### 17/8/5
		  cline1 = ents.add_cline(pt1,pt2)
		  cline1.end=nil
		  cline1.start=nil
		  cline1.stipple=cpatt
		  ###
		  theFace = entities.add_face(pt1_right, pt1_left, pt2_left, pt2_right) ###
		if vflag == 1 and height > 0
		   theFace.reverse!
		end
		if vflag == 1 and width < 0
		   theFace.reverse!
		end
		if vflag == 1 and pt1.z > pt2.z
		   theFace.reverse!
		end

		end
		### end sorting start / end of path ### 17/8/5 ###


		###
		### ask if face starts at correct side ? ### 7/8/5
		if @alignment == "边缘" or  @alignment == "中心(侧边)"
		  model.selection.clear
		  model.selection.add [theFace]
		  model.selection.add [theFace.edges]
		end

		### if NO then reverse face side ### 7/8/5

		### #################################################################

		@@theEdges= []
		  0.upto(@newVerts.length-2) do |i| ### 1/8/5
		  @@theEdges[i] = ents.add_line(@newVerts[i],@newVerts[i+1]) ### make vertices into edges
		end

		### follow me along selected edges

		if height > 0
		  if width > 0
			theExtrusion=theFace.reverse!.followme @@theEdges ###
		  end
		  if width < 0
			theExtrusion=theFace.followme @@theEdges ###
		  end
		end
		if height < 0
		  if width > 0
			theExtrusion=theFace.followme @@theEdges ###
		  end
		  if width < 0
			theExtrusion=theFace.reverse!.followme @@theEdges ###
		  end
		end

		###
		if not theExtrusion
		  UI.messagebox("无法挤压 ! \n所选路径可能有分叉 ! \n请重新选择单个连续路径... ")
		  entities.erase_entities theFace.edges
		  entities.erase_entities cline1
		  edge_reselector(@newVerts)
		  return nil
		end
		###
		entities.erase_entities cline1
		model.commit_operation

		### #################################################################
		### restore selection set of edges and display them
		edge_reselector(@newVerts)


		end ### end def
	end




############################################################# PRODUITS CUISINE NOUVELLE METHODE POUR CLICK-CHANGE #######################################################################
############################################################# PRODUITS CUISINE NOUVELLE METHODE POUR CLICK-CHANGE #######################################################################
################################################################### PLUGIN AF-Custom 2 - BARRES D'OUTILS ###########################################################################



def self.clica
	  definition = Sketchup.active_model.selection.first.definition
	  behavior = definition.behavior
	  behavior.no_scale_mask = 120
	  result = Sketchup.send_action "selectScaleTool:"
	  end


def plus_de_plugins
  Sketchup.send_action 10502;Sketchup.send_action 10519 
end

def self.select_unidx
    
	
	selection = Sketchup.active_model.selection
	unless selection.empty?
		
			selection.grep(Sketchup::ComponentInstance) do |e|
			$dc_observers.get_class_by_version(e).redraw_with_undo(e)
		end

end
		

    
	end

def self.options
	  if $dc_observers.get_latest_class.configure_dialog_is_visible
	  $dc_observers.get_latest_class.close_configure_dialog
	  else
	  $dc_observers.get_latest_class.show_configure_dialog
	  end
	  Sketchup.send_action("selectSelectionTool:")
	  end

def self.click
	  Sketchup.active_model.select_tool(DCInteractTool.new($dc_observers))
	  end
 def self.clicka
	  
	  Sketchup.send_action 21098
	  
	  end
	  
	  def self.clickb
	  
	  Sketchup.send_action 21237
	  
	  end
	  
	  
	  
def self.web

 UI.openURL("http://www.afu321.com")
end


def exec_on_autoload
  puts("cd_standard loaded.")
end


 def self.clica
	  definition = Sketchup.active_model.selection.first.definition
	  behavior = definition.behavior
	  behavior.no_scale_mask = 120
	  result = Sketchup.send_action "selectScaleTool:"
end

def self.afubox
	model = Sketchup.active_model
	abc_path =  Sketchup.find_support_file 'Plugins\af_Custom\components\box.skp'
	abc_def = model.definitions.load abc_path
	model.place_component(abc_def)
 end

 
 def self.afuqjbox
	model = Sketchup.active_model
	abc_path =  Sketchup.find_support_file 'Plugins\af_Custom\components\qjbox.skp'
	abc_def = model.definitions.load abc_path
	model.place_component(abc_def)
 end
 
  
 def self.afudjbox
	model = Sketchup.active_model
	abc_path =  Sketchup.find_support_file 'Plugins\af_Custom\components\djbox.skp'
	abc_def = model.definitions.load abc_path
	model.place_component(abc_def)
 end
   def self.afushitu
    sel = Sketchup.active_model.selection.grep(Sketchup::ComponentInstance)
      if sel.empty?||sel.size>1
        UI.messagebox("请选择一个组件进行操作")
      else
        c = sel[0]
        $distance ||= 200.mm
        prompts = ["输入视图间距"]
        defaults = [$distance]
        input = UI.inputbox(prompts, defaults, "三视图设置")
        if input
          $distance = input[0]
          trr = c.transformation
          xaxis = trr.xaxis
          bounds = c.bounds
        unless xaxis.samedirection? X_AXIS
          UI.messagebox("你的组件轴和场景的轴必须保持一致")
          return
          end
          pt = bounds.corner(0)
          pt1 = pt.offset(X_AXIS,-$distance)
          tr  = Geom::Transformation.new(pt1)
          rot = Geom::Transformation.rotation(pt1,Z_AXIS,90.degrees)
          Sketchup.active_model.entities.add_instance(c.definition,rot*tr)

          pt1 = pt.offset(X_AXIS,$distance+bounds.width)
          pt1 = pt1.offset(Y_AXIS,bounds.width)
          tr  = Geom::Transformation.new(pt1)
          rot = Geom::Transformation.rotation(pt1,Z_AXIS,-90.degrees)
          Sketchup.active_model.entities.add_instance(c.definition,rot*tr)

          pt1 = pt.offset(Z_AXIS,$distance+bounds.depth)
          pt1 = pt1.offset(Y_AXIS,bounds.depth)
          tr  = Geom::Transformation.new(pt1)
          rot = Geom::Transformation.rotation(pt1,X_AXIS,90.degrees)
          Sketchup.active_model.entities.add_instance(c.definition,rot*tr)
        end
      end
  end
############################################################# PRODUITS CUISINE NOUVELLE METHODE POUR CLICK-CHANGE #######################################################################


  if !file_loaded?(__FILE__)
  
    su_menu = UI.menu("Plugins")

    cc_menu = su_menu.add_submenu(%Q(TC-定制助手))

    tb = UI::Toolbar.new(%Q(TC-定制助手6.5/LAOYAO))

	
	
	
	
	#change_02 = UI::Command.new("切角格子"){Adc_sketchup::insert_TCZJ}

    #change_02.small_icon = File.join(@@path_to_resources,"add_30.png")

    #change_02.large_icon = File.join(@@path_to_resources,"add_30.png")

    #change_02.tooltip = %Q(切角格子)

    #change_02.status_bar_text = ("创建一个切角格子")

    #change_02.menu_text = ("切角格子")

    #cc_menu.add_item(change_02)

    
	#tb.add_item(change_02)
	
	
	
	
	
	
	#  change_18 = UI::Command.new("转角格子"){Adc_sketchup::insert_TCLB}

   # change_18.small_icon = File.join(@@path_to_resources,"add_32.png")

   # change_18.large_icon = File.join(@@path_to_resources,"add_32.png")

    #change_18.tooltip = %Q(转角格子)

    #change_18.status_bar_text = ("创建一个转角格子")

    #change_18.menu_text = ("转角格子")

    #cc_menu.add_item(change_18)

    #tb.add_item(change_18)
	  
	
	

	#change_16 = UI::Command.new("转角柜子"){Adc_sketchup::insert_TCCB}

    #change_16.small_icon = File.join(@@path_to_resources,"add_33.png")

    #change_16.large_icon = File.join(@@path_to_resources,"add_33.png")

    #change_16.tooltip = %Q(转角柜子)

   # change_16.status_bar_text = ("创建一个转柜子")

    #change_16.menu_text = ("转角柜子")

    #cc_menu.add_item(change_16)

   # tb.add_item(change_16)    
    
	
	
	
	 # change_17 = UI::Command.new("转角桌框"){Adc_sketchup::insert_TCHB}

    #change_17.small_icon = File.join(@@path_to_resources,"add_35.png")

   # change_17.large_icon = File.join(@@path_to_resources,"add_35.png")

    #change_17.tooltip = %Q(转角桌框)

    #change_17.status_bar_text = ("创建一个转角桌框")

    #change_17.menu_text = ("转角桌框")

    #cc_menu.add_item(change_17)

    #tb.add_item(change_17)      
    

	
	
	
	#tb.add_separator
   # @@change_20 = UI::Command.new("参数box"){self.afubox} 
	#{create_board}

  #  @@change_20.small_icon = File.join(@@path_to_resources,"rullers.png")

  #  @@change_20.large_icon = File.join(@@path_to_resources,"rullers.png")

  #  @@change_20.tooltip = %Q(参数box)

  #  @@change_20.status_bar_text = ("参数box")

  #  @@change_20.menu_text = ("参数box")
    
  #  @@change_20.set_validation_proc{MF_UNCHECKED}

  #  cc_menu.add_item(@@change_20)

 #   tb.add_item(@@change_20) 
	
	
	
	
	
		
	#tb.add_separator
    #@@change_20 = UI::Command.new("截面放样"){ProfileBuilderTool.create.show}
	#{create_board}

#    @@change_20.small_icon = File.join(@@path_to_resources,"rullers1.png")

 #   @@change_20.large_icon = File.join(@@path_to_resources,"rullers1.png")

  #  @@change_20.tooltip = %Q(切角盒子)

   # @@change_20.status_bar_text = ("切角盒子")

    #@@change_20.menu_text = ("切角盒子")
    
    #@@change_20.set_validation_proc{MF_UNCHECKED}

   # cc_menu.add_item(@@change_20)

   # tb.add_item(@@change_20) 
	
	
	
#	@@change_20 = UI::Command.new("构件放样"){FenceBuilderTool.create.show}

 #   @@change_20.small_icon = File.join(@@path_to_resources,"add12_l.png")

 #   @@change_20.large_icon = File.join(@@path_to_resources,"add12_l.png")

  #  @@change_20.tooltip = %Q(倒角盒子)

  #  @@change_20.status_bar_text = ("倒角盒子")

  #  @@change_20.menu_text = ("倒角盒子")
    
  #  @@change_20.set_validation_proc{MF_ENABLED}

  #  cc_menu.add_item(@@change_20)

  #  tb.add_item(@@change_20)
	
	
		
	
	
	
	
	#tb.add_separator
	
	
	
	
	
	
	#@@change_20 = UI::Command.new("取消粘接"){
  #sel = Sketchup.active_model.selection.grep(Sketchup::ComponentInstance)
#  unless sel.empty?
 # 	Sketchup.active_model.start_operation("Auto snapto")
  #	sel.each do |e|
  	#	b = e.definition.behavior
  	#	if b.is2d?
  	#		b.is2d = false
  	#	else
  	#		b.is2d = true
  	#		b.snapto = 0
  	#	end
  	#end
  	#Sketchup.active_model.commit_operation
  #else
  	#UI.messagebox("请先选择单个或者多个组件!")
  #end
#}

    #@@change_20.small_icon = File.join(@@path_to_resources,"add24_l.png")

    #@@change_20.large_icon = File.join(@@path_to_resources,"add24_l.png")

   # @@change_20.tooltip = %Q(取消粘接)

    #@@change_20.status_bar_text = ("取消粘接")

   # @@change_20.menu_text = ("取消粘接")
    
   # @@change_20.set_validation_proc{MF_UNCHECKED}

    #cc_menu.add_item(@@change_20)

   # tb.add_item(@@change_20) 
	
	

	
	
	#tb.add_separator
	
	
	
	
	
	
	#@@change_20 = UI::Command.new("绘制矩形") {Sketchup.active_model.select_tool RectangleTool.new }
	#{ ExtrudeAlongPath.main }

    #@@change_20.small_icon = File.join(@@path_to_resources,"add88_l.png")

    #@@change_20.large_icon = File.join(@@path_to_resources,"add88_l.png")

    #@@change_20.tooltip = %Q(绘制矩形)

    #@@change_20.status_bar_text = ("绘制矩形")

    #@@change_20.menu_text = ("绘制矩形")
    
    #@@change_20.set_validation_proc{MF_ENABLED}

    #cc_menu.add_item(@@change_20)

    #tb.add_item(@@change_20) 
	
	
	
	
	
	
	
	@@change_20 = UI::Command.new("面成实体"){ SteveR_toComponents.facestocoms(1) }

    @@change_20.small_icon = File.join(@@path_to_resources,"add12_l.png")

    @@change_20.large_icon = File.join(@@path_to_resources,"add12_l.png")

    @@change_20.tooltip = %Q(面成实体)

    @@change_20.status_bar_text = ("面成实体")

    @@change_20.menu_text = ("面成实体")
    
    @@change_20.set_validation_proc{MF_ENABLED}

    cc_menu.add_item(@@change_20)

    tb.add_item(@@change_20)
	
	
	
	

	
	#@@change_20 = UI::Command.new("查询长度"){ Sketchup.active_model.select_tool TrackMouseTool.new }

    #@@change_20.small_icon = File.join(@@path_to_resources,"net.png")

    #@@change_20.large_icon = File.join(@@path_to_resources,"net.png")

    #@@change_20.tooltip = %Q(查询长度)

    #@@change_20.status_bar_text = ("查询长度")

    #@@change_20.menu_text = ("查询长度")
    
    #@@change_20.set_validation_proc{MF_ENABLED}

    #cc_menu.add_item(@@change_20)

    #tb.add_item(@@change_20)
	
	
	
	@@change_20 = UI::Command.new("X光模式"){  Sketchup.send_action 10596 }

    @@change_20.small_icon = File.join(@@path_to_resources,"weathe.png")

    @@change_20.large_icon = File.join(@@path_to_resources,"weathe.png")

    @@change_20.tooltip = %Q(X光模式)

    @@change_20.status_bar_text = ("X光模式")

    @@change_20.menu_text = ("X光模式")
    
    @@change_20.set_validation_proc{MF_ENABLED}

    cc_menu.add_item(@@change_20)

    tb.add_item(@@change_20)
	
	
	
	
	
	
	
	
	
	 @@change_20 = UI::Command.new("刷新模型"){ self.select_unidx }

    @@change_20.small_icon = File.join(@@path_to_resources,"add9_l.png")

    @@change_20.large_icon = File.join(@@path_to_resources,"add9_l.png")

    @@change_20.tooltip = %Q(刷新模型)

    @@change_20.status_bar_text = ("刷新模型")

    @@change_20.menu_text = ("刷新模型")
    
    @@change_20.set_validation_proc{MF_UNCHECKED}

    cc_menu.add_item(@@change_20)

    tb.add_item(@@change_20) 
	
	
#	tb.add_separator
	
	
	
	
	
	
	@@change_20 = UI::Command.new("未用清理"){ 
	
	Sketchup.active_model.definitions.purge_unused
	Sketchup.active_model.materials.purge_unused

	UI.messagebox('未使用的组件和材质清理成功')
	}

    @@change_20.small_icon = File.join(@@path_to_resources,"easyicon.net.png")

    @@change_20.large_icon = File.join(@@path_to_resources,"easyicon.net.png")

    @@change_20.tooltip = %Q(未用清理)

    @@change_20.status_bar_text = ("未用清理")

    @@change_20.menu_text = ("未用清理")
    
    @@change_20.set_validation_proc{MF_ENABLED}

    cc_menu.add_item(@@change_20)

    tb.add_item(@@change_20) 
	
	#@@change_20 = UI::Command.new("跟随开口"){holepunch()}
	#{ TIG::HolePunchTool.new() }

   # @@change_20.small_icon = File.join(@@path_to_resources,"add3_l.png")

   # @@change_20.large_icon = File.join(@@path_to_resources,"add3_l.png")

   # @@change_20.tooltip = %Q(跟随开口)

   # @@change_20.status_bar_text = ("跟随开口")

   # @@change_20.menu_text = ("跟随开口")
    
   # @@change_20.set_validation_proc{MF_ENABLED}

   # cc_menu.add_item(@@change_20)

   # tb.add_item(@@change_20) 
	#tb.add_separator
	
	
	
		
	tb.add_separator
	
	
	
	
	@@change_20 = UI::Command.new("多重切口"){ (AF::Custom.start_trim_tool) }

    @@change_20.small_icon = File.join(@@path_to_resources,"add4_l.png")

    @@change_20.large_icon = File.join(@@path_to_resources,"add4_l.png")

    @@change_20.tooltip = %Q(多重切口)

    @@change_20.status_bar_text = ("多重切口")

    @@change_20.menu_text = ("多重切口")
    
    @@change_20.set_validation_proc{MF_ENABLED}

    cc_menu.add_item(@@change_20)

    tb.add_item(@@change_20) 
	
	
	
	
	
	
	
	
	
	
	
	
	
	@@change_20 = UI::Command.new("增强拉伸"){
			Sketchup.active_model.select_tool( Select_Through_Tool.new )
		}

    @@change_20.small_icon = File.join(@@path_to_resources,"add7_l.png")

    @@change_20.large_icon = File.join(@@path_to_resources,"add7_l.png")

    @@change_20.tooltip = %Q(增强拉伸)

    @@change_20.status_bar_text = ("谨用！他是强制拉伸，会破坏“动态组件”的参数，拉伸后参数不可二次调整")

    @@change_20.menu_text = ("谨用！他是强制拉伸，会破坏“动态组件”的参数，拉伸后参数不可二次调整")
    
    @@change_20.set_validation_proc{MF_UNCHECKED}

    cc_menu.add_item(@@change_20)

    tb.add_item(@@change_20) 
	
	
	
	
	@@change_20 = UI::Command.new("轴点拉伸"){ self.clica }

    @@change_20.small_icon = File.join(@@path_to_resources,"add17_l.png")

    @@change_20.large_icon = File.join(@@path_to_resources,"add17_l.png")

    @@change_20.tooltip = %Q(轴点拉伸)

    @@change_20.status_bar_text = ("轴点拉伸")

    @@change_20.menu_text = ("轴点拉伸")
    
    @@change_20.set_validation_proc{MF_UNCHECKED}

    cc_menu.add_item(@@change_20)

    tb.add_item(@@change_20) 
	
	
	
	
	
	
	@@change_20 = UI::Command.new("3点镜像"){Sketchup.active_model.select_tool(MirrorTool.new())}

    @@change_20.small_icon = File.join(@@path_to_resources,"MI.png")

    @@change_20.large_icon = File.join(@@path_to_resources,"MI.png")

    @@change_20.tooltip = %Q(3点镜像)

    @@change_20.status_bar_text = ("3点镜像")

    @@change_20.menu_text = ("3点镜像")
    
    @@change_20.set_validation_proc{MF_UNCHECKED}

    cc_menu.add_item(@@change_20)

    tb.add_item(@@change_20)
	
	

			
	
	
	change_17 = UI::Command.new("快速靠齐"){
		Sketchup.active_model.select_tool( Throw_afu_face.new )
	}

    change_17.small_icon = File.join(@@path_to_resources,"add_34.png")

    change_17.large_icon = File.join(@@path_to_resources,"add_34.png")

    change_17.tooltip = %Q(快速靠齐)

    change_17.status_bar_text = ("选择对象，点击要靠齐的面")

    change_17.menu_text = ("快速靠齐")

    cc_menu.add_item(change_17)

    tb.add_item(change_17) 
	
	
	
	
	
	
    tb.add_separator
	
	
	
		
	
	
	change_02 = UI::Command.new("前视正交"){    Sketchup.send_action 10502;Sketchup.send_action 10519 }

    change_02.small_icon = File.join(@@path_to_resources,"front.png")

    change_02.large_icon = File.join(@@path_to_resources,"front.png")

    change_02.tooltip = %Q(前视正交)

    change_02.status_bar_text = ("前视正交")

    change_02.menu_text = ("前视正交")

    cc_menu.add_item(change_02)

    
	tb.add_item(change_02)
	
	
	
	
	
		change_19 = UI::Command.new("生成视图"){self.afushitu}

		change_19.small_icon = File.join(@@path_to_resources,"four_view.png")

		change_19.large_icon = File.join(@@path_to_resources,"four_view.png")
	
		change_19.tooltip = %Q(生成视图)

		change_19.status_bar_text = ("生成视图")

		change_19.menu_text = ("生成视图")

		cc_menu.add_item(change_19)

		tb.add_item(change_19)
    
	
	
	
	 change_02 = UI::Command.new("输出二维图像"){Sketchup.send_action 21237}

    change_02.small_icon = File.join(@@path_to_resources,"dwg.png")

    change_02.large_icon = File.join(@@path_to_resources,"dwg.png")

    change_02.tooltip = %Q(输出二维图像)

    change_02.status_bar_text = ("输出二维图像")

    change_02.menu_text = ("输出二维图像")

    cc_menu.add_item(change_02)

    
	tb.add_item(change_02)
	
	
	
	    tb.add_separator
	
	
	

	@@change_20 = UI::Command.new("参数面板"){ options }

    @@change_20.small_icon = File.join(@@path_to_resources,"add_l.png")

    @@change_20.large_icon = File.join(@@path_to_resources,"add_l.png")

    @@change_20.tooltip = %Q(参数面板)

    @@change_20.status_bar_text = ("参数面板")

    @@change_20.menu_text = ("参数面板")
    
    @@change_20.set_validation_proc{MF_UNCHECKED}

    cc_menu.add_item(@@change_20)

    tb.add_item(@@change_20) 
	
	
	
	@@change_20 = UI::Command.new("对象交互"){ self.click}

    @@change_20.small_icon = File.join(@@path_to_resources,"add5_l.png")

    @@change_20.large_icon = File.join(@@path_to_resources,"add5_l.png")

    @@change_20.tooltip = %Q(对象交互)

    @@change_20.status_bar_text = ("对象交互")

    @@change_20.menu_text = ("对象交互")
    
    @@change_20.set_validation_proc{MF_UNCHECKED}

    cc_menu.add_item(@@change_20)

    tb.add_item(@@change_20) 
	
	
	#tb.add_separator



	  change_23 = UI::Command.new("规划算量") {Plugin.instance.show_dialog('materials')}

    change_23.small_icon = File.join(@@path_to_resources,"add_20.png")

    change_23.large_icon = File.join(@@path_to_resources,"add_20.png")

    change_23.tooltip = %Q(规划算量)

    change_23.status_bar_text = ("规划算量")

    change_23.menu_text = ("规划算量")

    cc_menu.add_item(change_23)

    tb.add_item(change_23)  


change_23 = UI::Command.new("帮助更新"){
   self.web
	#AFU = UI::HtmlDialog.new(
			#UI.openURL("http://www.afu321.com/")
			#{
			#  :dialog_title => "-AFU-全屋定制",
			#  :preferences_key => "com.sample.plugin",
			#  :scrollable => true,
			#  :resizable => true,
			#  :width => 600,
			#  :height => 400,
			#  :left => 100,
			#  :top => 100,
			#  :min_width => 335,
			#  :min_height => 800,
			#  :max_width =>1000,
			#  :max_height => 1000,
			#  :style => UI::HtmlDialog::STYLE_DIALOG
			#	})
			#	AFU.set_url("http://www.afu321.cn")
			#	AFU.show

  # AFU = UI::WebDialog.new("-AFU-", true,
   #  "-AFU-", 739, 641, 150, 150, true);
    # AFU.set_url "http://www.afu321.cn"
    #AFU.show 

}

    change_23.small_icon = File.join(@@path_to_resources,"change_23.png")

    change_23.large_icon = File.join(@@path_to_resources,"change_23.png")

    change_23.tooltip = %Q(在线模型库!)

    change_23.status_bar_text = ("浏览网站，模型，插件,教程...!")

    change_23.menu_text = ("浏览网站，模型，插件,教程...!")

   cc_menu.add_item(change_23)

    tb.add_item(change_23)  


 		 
    tb.restore

    file_loaded(__FILE__)
	  end
  end
end
end
end