
# License Module Injection Start

module AF::Custom

# Constants below must be edited using the License Module Build Script

EXT_ID = 'not-implemented'.freeze

TRIAL_DAYS = 0.freeze

LIC_FILE = 'booltools.lic'.freeze

LIC_DIR = 'Custom'.freeze

ACTIVATE_URL = 'http://mindsightstudios.com/lic/activate.php'.freeze

require 'time'

require 'digest'



module Licensing



  NOT_LICENSED = 100.freeze

  LICENSED = 101.freeze

  TRIAL = 102.freeze

  TRIAL_EXPIRED = 103.freeze

  LICENSE_ERROR = 104.freeze

  # NOTE: The constant 'PLUGIN' must be defined elsewhere and must

  # reference the parent module (which should always be the containing

  # module for this plugin. eg: AF::Artisan)



  # NOTE: The constant 'PLUGIN_NAME' must also be defined elsewhere and should reference

  # the standard display name for the plugin. (eg. 'Artisan' or 'Profile Builder 2')



  # NOTE: The constant 'PLUGIN::PATH' must be defined elsewhere and should reference

  # the folder that contains the main ruby files for this plugin



  # NOTE: The constant 'PLUGIN::VERSION' must be defined elsewhere and should reference

  # the current version number for this plugin (eg. 1.2.3)



  # NOTE: The constant 'PLUGIN::LIC_DIR' must be defined elsewhere and should reference

  # the directory name for the license file for this plugin



  # NOTE: The constant 'PLUGIN::LIC_FILE' must be defined elsewhere and should reference

  # the name for the license file for this plugin (eg. pb2.lic)



  # NOTE: The constant 'PLUGIN::ACTIVATE_URL' must be defined elsewhere and should reference

  # the license activation url for this plugin



  def self.get_app_data

    temp_dir = ON_MAC ? (File.join(ENV['HOME'], 'Library', 'Application Support')) : ((ENV['ALLUSERSPROFILE']))

    app_data = File.join(temp_dir, LIC_DIR)

    return app_data

  end



  MAC_LOCATION = ENV['systemroot'] ? File.join(ENV['systemroot'], 'System32') : '' #systemroot is a read-only variable so we will use it instead of windir

  ON_MAC = Sketchup.platform == :platform_osx

  APP_DATA = self.get_app_data

  Dir.mkdir(APP_DATA) unless FileTest.exist?(APP_DATA)

  require 'win32ole' unless ON_MAC



  module WebDialogHelper



    def show_dialog(&block)

      if ON_MAC == true

        @dlg.show_modal(&block)unless @dlg.visible?

      else

        @dlg.show(&block) unless @dlg.visible?

      end

    end



    def add_slashes(text)

      val = ""

      if text

        val = text.dup

        val.gsub!(/\\/, "\\\\\\")

        val.gsub!(/\n/, "\\\\n")

        val.gsub!(/"/, "\\\"")

      end

      val

    end



    def setValue(id, value, dlg = @dlg)   #sets the value of an input field in the web dialog

      new_value = add_slashes(value.to_s)

      cmd = "document.getElementById('#{id}').value=\"#{new_value}\""

      dlg.execute_script(cmd)

    end



    def call_js(function, *arguments)

      js_args = arguments.map { |x| x.to_json }.join(", ")

      javascript = "#{function}(#{js_args});"

      @dlg.execute_script(javascript)

    end



  end #webdialog helper



  class LicenseInfo



    @@info = nil

    include Licensing::WebDialogHelper



    def self.instance

      @@info = new unless @@info

      return @@info

    end



    #PLUGIN set to plugin module (Artisan, PB2 ...)

    def initialize

      @lic = License.new

      @lic.check?

      dlg_name = "许可证管理器: #{PLUGIN_NAME}"

      dlg_id = "#{PLUGIN_NAME}LicInfo"

      @dlg = UI::WebDialog.new(dlg_name, false, dlg_id, 450, 380, 150, 150, true)

      @dlg.set_file(File.join(PLUGIN::PATH, 'license', 'lic_info.html'))

      color = @dlg.get_default_dialog_color

      @dlg.set_background_color(color)

      @dlg.add_action_callback('ValueChanged') { |d, p|

        pair = p.split("|")

        id = pair[0]

        case id

          when "addLicense"

            if @lic.get_state == LICENSED

              UI.messagebox("您已拥有有效的许可证。")

            else

              @lic.register {refresh_dialog}

            end

          when "removeLicense"

            mss_lic_state = @lic.get_state

            if mss_lic_state == TRIAL or mss_lic_state == TRIAL_EXPIRED

              result = UI.messagebox("您确定要删除试用许可证吗？", MB_YESNO)

              if result == 6

                @lic.remove_license

                refresh_dialog

              end

            else

              UI.messagebox("只能删除MindSight Studios提供的试用许可证。")

            end

        end

      }

    end #init



    def show

      show_dialog {refresh_dialog}

    end



    def refresh_dialog

      @lic = License.new

      @lic.check?

      product = @lic.product + " (#{PLUGIN::VERSION})"

      set_visibility(@lic.licensor)

      setValue("product", product)

      setValue("customer", @lic.customer)

      setValue("email", @lic.email)

      setValue("expires", @lic.expires)

      setValue("type", @lic.lic_type)

      setValue("for", @lic.lic_for)

      setValue("licensor", @lic.licensor)

      setValue("status", get_friendly_license_state_name)

    end



    def set_visibility(licensor)

      if licensor == License::COMPANY

        cmd = "$('.extra').show();"

      else

        cmd = "$('.extra').hide();"

      end

      @dlg.execute_script(cmd)

    end



    def get_friendly_license_state_name

      name = ""

      case @lic.status

        when LICENSED

          name = "Licensed"

        when TRIAL

          name = "Trial (#{@lic.days_remaining} days remaining)"

        when TRIAL_EXPIRED

          name = "Trial Expired"

        when NOT_LICENSED

          name = "Not Licensed"

      end



      name

    end



  end #LicenseInfo



  class License



    include Licensing::WebDialogHelper



    COMPANY = "MindSight Studios Inc."

    EW = "SketchUp Extension Warehouse"

    MAC_ADDRESS_TEMP_FILE = File.join(Sketchup.temp_dir, "dm_#{PLUGIN_NAME}_mac.txt".downcase)



    def self.write_mac_address_temp_file

      if !ON_MAC

        params = File.exists?(MAC_LOCATION) ? "/c cd #{MAC_LOCATION} && " : '/c ' #change to normal ipconfig.exe location if folder exists

        params += "ipconfig.exe /all > \"#{MAC_ADDRESS_TEMP_FILE}\"" #add the ipconfig command that will execute (either we are in the correct folder or hopefully the system path is properly set)

        shell = WIN32OLE.new('Shell.Application')

        shell.ShellExecute('cmd.exe', params, '', 'open', 0)

      end

    end



    def self.file

      File.join(Licensing::APP_DATA, LIC_FILE)

    end



    def self.wait_for_file_create

      License.write_mac_address_temp_file

      step = 0.05

      count = 0.0

      while (count < 5.0) do

        if FileTest.size?(MAC_ADDRESS_TEMP_FILE) && !FileTest.zero?(MAC_ADDRESS_TEMP_FILE)

          sleep(step)

          return

        end

        sleep(step)

        count += step

      end

      puts "Mac地址临时文件创建超时"

    end





    def self.remove

      lic = License.new

      status = lic.remove_license

      if status == 1

        return true

      else

        return false

      end

    end



    def initialize

      @ext_id = "#{EXT_ID}"

      @lic = License.file

      @licensor = ""

      @product = "#{PLUGIN_NAME}"

      @customer = "Unregistered User"

      @email = ""

      @type = "" #single user, network, trial

      @lic_for = "" #trial, commercial, educational, nfr

      @expires = ""

      @status = ""

      @days_remaining = 0

      @mac_address_array = get_mac_address

      @magic_number = "29678692200"

      @sig = ""

      @callback = nil

      dlg_name = "Activate License: " + PLUGIN_NAME

      dlg_id = "Reg#{PLUGIN_NAME}"

      if ON_MAC

        @dlg = UI::WebDialog.new(dlg_name, false, dlg_id, 600, 200,

                                 150, 150, true)

      else

        @dlg = UI::WebDialog.new(dlg_name, false, nil, 600, 200,

                                 150, 150,true)

      end

      @dlg.set_url(ACTIVATE_URL)

      color = @dlg.get_default_dialog_color

      @dlg.set_background_color(color)

      @dlg.add_action_callback("create_license") { |d, p|

        lic_info = {}

        lic_info["licensee"] = d.get_element_value("licensee")

        lic_info["email"] = d.get_element_value("lic_email")

        lic_info["lic_type"] = d.get_element_value("lic_type")

        lic_info["lic_for"] = d.get_element_value("lic_for")

        lic_info["expires"] = d.get_element_value("expires")

        lic_info["product"] = d.get_element_value("product")

        lic_info["license_key"] = d.get_element_value("license_key")

        lic_info["num_licenses"] = d.get_element_value("num_licenses")

        lic_info["num_activations"] = d.get_element_value("num_activations")

        lic_info["entitled_activations"] = d.get_element_value("entitled_activations")

        status = validate_license(lic_info)

        if status

          lic_for = lic_info["lic_for"]

          num_activations = lic_info["num_activations"].to_i

          num_licenses = lic_info["num_licenses"].to_i

          if lic_for == "Trial"

            UI.messagebox("发现试用许可证。 感谢您的关注" +

                              " 在 #{@product}!")

          else

            UI.messagebox("谢谢你的购买 #{@product}!")

            p "许可证:#{num_licenses} - 激活: #{num_activations + 1}"

          end

          create_license(lic_info)

          @callback.call if @callback

          @callback = nil

        end

        d.close

      }

      @dlg.add_action_callback("lic_not_found") { |d, p|

        d.close

        UI.messagebox("抱歉，您的许可证密钥似乎无效。\n请再试一次。 " +

                          "一定要使用复制/粘贴（CTRL + C和CTRL + V）进入" +

                          "许可证信息电子邮件中的密钥。")

      }

      @dlg.add_action_callback("entitlement_exceeded") { |d, p|

        d.close

        UI.messagebox("您已超过允许的激活次数" +

                          " 对于此许可证。 请联系客户支持。")

      }

      # A very strange issue occurred on OSX when this callback was named 'set_product'

      #  The callback was being executed when 'initialize' was called for this class.

      #  But, when the method name was changed to 'set_the_product', this behavior

      #  no longer happened.

      #  This callback was needed for OSX as a workaround to an issue where the block

      #  passed to 'show' was not being executed for some reason, possibly because this

      #  is a remote dialog

      @dlg.add_action_callback("set_the_product") { |d, p|

        set_product

      }

    end



    # Over-ride inherited Object methods to help protect class

    def extend(*args)

    end

    def inspect

      ""

    end

    def instance_eval(*args)

    end

    def instance_variables

      []

    end

    def method(*args)

    end

    def methods

      []

    end

    def private_methods

      []

    end

    def public_methods

      []

    end

    def protected_methods

      []

    end

    def respond_to?(*args)

    end

    def send(*args)

    end

    def singleton_methods

      []

    end

    def instance_methods

      []

    end

    def public_instance_methods

      []

    end

    def protected_instance_methods

      []

    end

    def private_instance_methods

      []

    end

    def to_a

      []

    end

    def to_s

      ""

    end

    def constants

      []

    end

    def class_variables

      []

    end



    public

    def remove_license

      #TODO - decrement number of activations in server?

      num_files = 0

      if FileTest.exist?(@lic)

        num_files = File.delete(@lic)

      end

      num_files

    end



    def check?
valid_license = true
@status=LICENSED
@licensor="abc.163.com"	
return valid_license
      valid_license = false

      lic_status = get_state

      @status = lic_status



      case lic_status

        when LICENSED

          valid_license = true

        when TRIAL

          @days_remaining = get_rem_eval_days

          valid_license = true

        when TRIAL_EXPIRED

        when NOT_LICENSED

          @licensor = ""

      end

      valid_license

    end



    def get_state
status = LICENSED
return status
      status = nil

      if read_license_file

        if has_valid_license?

          @licensor = COMPANY

          if @lic_for != "Trial"

            status = LICENSED

          elsif get_rem_eval_days > 0.0

            status = TRIAL

            days_remaining = get_rem_eval_days

          else

            status = TRIAL_EXPIRED

          end

        else

          status = NOT_LICENSED

        end

      else

        status = NOT_LICENSED

      end

      status

    end



    def register(&block)

      if Sketchup.is_online

        @callback = block

        @dlg.show_modal {set_product}

      else

        UI.messagebox("没有互联网连接。 " +

                          "连接到互联网以激活您的许可证。")

      end

    end



    def product

      @product.dup

    end



    def customer

      @customer.dup

    end



    def email

      @email.dup

    end



    def expires

      @expires.dup

    end



    def lic_type

      @type.dup

    end



    def lic_for

      @lic_for.dup

    end



    def licensor

      @licensor.dup

    end



    def days_remaining

      @days_remaining

    end



    def status

      @status

    end



    private

    def validate_license(lic_info)

      num_activations = lic_info["num_activations"].to_i

      num_licenses = lic_info["num_licenses"].to_i

      entitled_activations = lic_info["entitled_activations"].to_i

      valid = true



      if num_activations == entitled_activations

        UI.messagebox("您已超过允许的激活次数" +

                          " 对于此许可证。 请联系客户支持。")

        valid = false

      end



      valid

    end



    def create_license(lic_info)

      @customer = lic_info["licensee"]

      @email = lic_info["email"]

      @type = lic_info["lic_type"]

      @lic_for = lic_info["lic_for"]

      @expires = lic_info["expires"]

      create_license_file

    end



    def get_rem_eval_days

      t_now = Time.now

      t_now.gmtime

      ymd = @expires.split("-")

      year = ymd[0]

      month = ymd[1]

      day = ymd[2]

      exp_time = Time.utc(year, month, day)



      remaining_seconds = exp_time - t_now

      remaining_days = remaining_seconds.to_f / (24 * 3600)

      remaining_days = remaining_days.ceil



      remaining_days

    end



    def set_product

      setValue('productSU', PLUGIN_NAME, @dlg)

    end



    def get_license

      read_license_file

    end



    def has_valid_license?

      @mac_address_array = get_mac_address



      @mac_address_array.each { |mac|

        return true if @sig == get_sig(mac)

      }

      false

    end



    def read_license_file

      status = false

      if FileTest.exist?(@lic) && !FileTest.zero?(@lic)

        begin

          f = File.new(@lic, "r")

          lines = f.read.split("\n")

          f.close

          data = lines[1].split(";")

          @product = data[0]

          @customer = data[1]

          @email = data[2]

          @type = data[3]

          @lic_for = data[4]

          @expires = data[5]

          @sig = lines[2]

          status = true

        rescue

          UI.messagebox("读取 #{@product}  许可证文件时出错。")

          status = false

        end

      elsif FileTest.zero?(@lic)

        File.delete(@lic)

      end



      status

    end



    def get_sig(mac)

      str = [@product, @customer, @email, @type, @lic_for, @expires, mac,

             @magic_number].join

      sig = Digest::MD5.hexdigest(str)

      sig

    end



    def create_license_file

      begin

        out = File.new(@lic, "w")

        write_license(out)

        out.close

      rescue

        UI.messagebox("创建许可文件时出错。")

        # TODO(Dont' raise here for public release)

        #raise

      end

    end



    def write_license(out)

      mac_address_length = 17 #00:00:00:00:00:E0

      if (@mac_address_array[0].length != mac_address_length)

        UI.messagebox("无法创建许可证文件。 请联系客户支持。")

        #TODO need to refund the user an activation?

      else

        data = [@product, @customer, @email, @type, @lic_for, @expires]

        out.puts "LICENSE"

        out.puts data.join(";")

        out.puts get_sig(@mac_address_array[0])

      end

    end



    def get_mac_address

      return @mac_address_array if @mac_address_array



      lines = []



      if ON_MAC

        cmd = 'ifconfig'

        lines = %x[#{cmd}].split("\n")

      else

        if FileTest.exist?(MAC_ADDRESS_TEMP_FILE)

          #puts "Mac address temp file exists"

        else

          License.wait_for_file_create

        end

        file = File.open(MAC_ADDRESS_TEMP_FILE, 'r', :encoding => Encoding::find("filesystem"))

        lines = file.read.split("\n")

        file.close

      end



      mac = parse_mac_address(lines)

      mac

    end



    def parse_mac_address(lines)

      macs = Array.new

      regex = Regexp.compile('^.*\s(?<mac>([0-9a-f|A-F]{2}[:-]){5}([0-9a-f|A-F]{2}))$')



      state = 0

      tmp   = ""



      lines.each { |line|

        next if (line.nil? || line.empty?)



        # for consistency we need to convert - to : and make upper case and strip spaces of right side

        test = line.gsub(/-/, ':').upcase().rstrip()



        # returns last 17 characters of mac address - might be a DHCPv6

        tmp = regex.match(test)

        tmp = tmp['mac'].to_s if tmp

        found = (tmp != nil && tmp != "")

        macs.push(tmp) if (found)

      }



      macs

    end



    freeze



  end #class License



  License.write_mac_address_temp_file

  freeze #module Licensing



end #module Licensing





end

# This file is generated from the build_license.rb script. 

# Do not edit this file manually.



#require 'sketchup.rb'

module AF::Custom



    #def self.get_icon(name)

 #     ext = Sketchup.platform == :platform_osx ? '.pdf' : '.svg'

#      File.join(File.dirname(__FILE__), 'Icons', name + ext)

  #  end



   # def self::build_ui

     # tool_bar_icons = {

          #union: self.get_icon('union'),

         # trim: self.get_icon('trim'),

          #differnce: self.get_icon('subtract'),

          #split: self.get_icon('split'),

          #intersect: self.get_icon('intersect'),

          #self_union: self.get_icon('self_union'),

          #cart: self.get_icon('cart')

     # }



      #tb = UI::Toolbar.new("a2233")

      #cmd1 = UI::Command.new("1并集") { (AF::Custom.start_union_tool) }

      #cmd1.small_icon = tool_bar_icons[:union]

      #cmd1.large_icon = tool_bar_icons[:union]

      #cmd1.tooltip = "2并集"

      #cmd1.status_bar_text = "选择两个实体进行并集运算，将其合并为一个实体。"



      #cmd2 = UI::Command.new("差集") { (AF::Custom.start_difference_tool) }

      #cmd2.small_icon = tool_bar_icons[:differnce]

      #cmd2.large_icon = tool_bar_icons[:differnce]

      #cmd2.tooltip = "差集2"

      #cmd2.status_bar_text = "从第一个实体中减去第二个选定的实体。"



      #cmd3 = UI::Command.new("交集") { (AF::Custom.start_intersection_tool) }

      #cmd3.small_icon = tool_bar_icons[:intersect]

      #cmd3.large_icon = tool_bar_icons[:intersect]

      #cmd3.tooltip = "交集3"

      #cmd3.status_bar_text = "选择两个实体进行交集运算，并保留相交部分。"



      #cmd4 = UI::Command.new("修剪") { (AF::Custom.start_trim_tool) }

      #cmd4.small_icon = tool_bar_icons[:trim]

      #cmd4.large_icon = tool_bar_icons[:trim]

      #cmd4.tooltip = "修剪4"

      #cmd4.status_bar_text = "从第一个实体减去第二个选定的实体，并将两者都保留在模型中。"



      #cmd5 = UI::Command.new("分离") { (AF::Custom.start_split_tool) }

      #cmd5.small_icon = tool_bar_icons[:split]

      #cmd5.large_icon = tool_bar_icons[:split]

      #cmd5.tooltip = "分离5"

      #cmd5.status_bar_text = "选择两个实体进行交集运算，并保留全部结果。"



      #cmd6 = UI::Command.new("组内合并") { (AF::Custom.start_self_union_tool) }

     # cmd6.small_icon = tool_bar_icons[:self_union]

      #cmd6.large_icon = tool_bar_icons[:self_union]

     # cmd6.tooltip = "组内合并"

    #  cmd6.status_bar_text = "单击一个群组或组件，将群组或组件内部所有嵌套实体合并为一个实体。"



      #cmd7 = UI::Command.new("购买许可证") { (AF::Custom.purchase_license) }

      #cmd7.small_icon = tool_bar_icons[:cart]

      #cmd7.large_icon = tool_bar_icons[:cart]

      #cmd7.tooltip = "购买 #{PLUGIN_NAME}"

      #cmd7.status_bar_text = "购买许可证 #{PLUGIN_NAME}"



      #cmd8 = UI::Command.new("**许可证..") { Licensing::LicenseInfo.new.show }

     # cmd8.tooltip = cmd8.status_bar_text = "显示许可信息"



      #tb.add_item(cmd1)

      #tb.add_item(cmd2)

      #tb.add_item(cmd3)

      #tb.add_item(cmd4)

      #tb.add_item(cmd5)

      #tb.add_item(cmd6)



      #unless FileTest.exist?(Licensing::License.file)

      #  tb.add_separator

      #  tb.add_item(cmd7)

      #end



      # This is need on OSX so that toolbar visibility settings will persist

      #case tb.get_last_state

      #  when -1, 1 #never been shown or visible

       #   tb.show

       # else

       #   tb.hide

     # end



      #tool_menu = UI.menu("Plugins")

     # main_menu = tool_menu.add_submenu("2233")

      #main_menu.add_item(cmd1)

      #main_menu.add_item(cmd2)

     # main_menu.add_item(cmd3)

     # main_menu.add_item(cmd4)

     # main_menu.add_item(cmd5)

     # main_menu.add_item(cmd6)

      #main_menu.add_separator

      #main_menu.add_item(cmd8)

   # end



  def self.start_difference_tool

    Sketchup.active_model.select_tool(BoolDifferenceTool.new)

  end



  #def self.start_self_union_tool

  #  Sketchup.active_model.select_tool(BoolSelfUnionTool.new)

 # end



  def self.start_trim_tool

    Sketchup.active_model.select_tool(BoolTrimTool.new)

  end



  def self.start_split_tool

    Sketchup.active_model.select_tool(BoolSplitTool.new)

  end



  def self.start_union_tool

    Sketchup.active_model.select_tool(BoolUnionTool.new)

  end



  def self.start_intersection_tool

    Sketchup.active_model.select_tool(BoolIntersectionTool.new)

  end



  # Since we are freezing the main plugin module, we need to set these

  # variables when SketchUp loads the script since they cannot

  # be dyanmically set after the module is frozen

  @dlg = UI::WebDialog.new("Buy Custom", true,

                           "Buy-Custom", 800, 800, 150, 150, true)

  @dlg.set_url("http://mindsightstudios.com/booltools/buy")



  def self.purchase_license

    if Sketchup.platform == :platform_osx

      @dlg.show_modal

    else

      @dlg.show

    end

  end



  #unless file_loaded?(__FILE__)

   # self::build_ui

  #  file_loaded(__FILE__)

 # end



end
 #module Custom
