
require 'sketchup.rb'

module AF::Custom



  MODIFIED_DICTIONARY = 'AF BoolTool Modified Definition'

  MODIFIED_DIC_KEY = 'Modified'



  #any instance sent will have an attribute/dictionary set on its definition marking it as a changed instance

  def set_bool_tools_modified_attribute(instances)

    definitions = instances.collect{ |i| i.definition }.uniq

    definitions.each{ |defin|

      defin.set_attribute(MODIFIED_DICTIONARY, MODIFIED_DIC_KEY, true)

    }

  end



  #any instance definition that has the attribute/dictionary wont after this completes

  #this prevents an object being checked every time a save is complete and only happens for the objects changed since the last save

  def clear_bool_tools_modifies_attribute(model)

    model.definitions.collect{ |d|

      if (d.get_attribute(MODIFIED_DICTIONARY, MODIFIED_DIC_KEY))

        d.delete_attribute(MODIFIED_DICTIONARY) #remove the dictionary

      end

    }

  end



  class BTAppObserver < Sketchup::AppObserver

    def onNewModel(model)

      model.add_observer(BTModelObserver.new)

    end



    def onOpenModel(model)

      model.add_observer(BTModelObserver.new)

    end

  end



  class BTModelObserver < Sketchup::ModelObserver

    include AF::Custom



    def initialize(*args)

      @start_operation_ok = 1

      super

    end



    def onPreSaveModel(model)

      prompt_user_for_scaling_to_prevent_su_cleanup(model)

      @start_operation_ok = 1

    end

#

#     #todo made behaviour of saving components inconsistent so removed for now

#     # # @param [ComponentInstance] model, SU function that sends componentInstance not a model

#     # def onBeforeComponentSaveAs(model)

#     #   prompt_user_for_scaling_to_prevent_su_cleanup

#     # end

#

    def prompt_user_for_scaling_to_prevent_su_cleanup(model = Sketchup.active_model)

      if (@start_operation_ok == 1)

        @start_operation_ok = 2

        #check if SU will want to alter the resulting objects

        #prompt user if they want to use transformation to avoid SU from modifying the result

        modified_instances = model.definitions.collect{ |d| d.instances.first if (d.get_attribute(MODIFIED_DICTIONARY, MODIFIED_DIC_KEY))}.compact

        unless (Boolean.result_is_valid?(modified_instances))

          res = UI.messagebox("哎呀！\n\n看起来您的布尔工具操作时创建了一些非常小的边缘，SketchUp会在保存模型时尝试删除这些边缘。"+

        "\n\n您是否希望布尔工具应用魔术缩放以防止在SketchUp中的您的实体模型产生破面？"+"\n\n此功能对于过于复杂的模型不一定有效！", MB_YESNO)

          if (res == IDYES)

            model.start_operation('Custom Magic Scaling', true)

            Boolean.avoid_tiny_edges(modified_instances)

            model.commit_operation

          end

        end

        unless (modified_instances.empty?)

          model.start_operation('Clear BoolTool dictionaries', true)

          clear_bool_tools_modifies_attribute(model)

          model.commit_operation

        end

      end

    end



  end



  # Attach the observer

  Sketchup.add_observer(BTAppObserver.new)

  Sketchup.active_model.add_observer(BTModelObserver.new)



end