
module AF::Custom



  class BTPreviewer < Sketchup::ViewObserver



    CAGE_MATERIAL_NAME = "布尔工具预览界限"

    PREVIEW_ALPHA = 230 # 0 is opaque, 255 is transparent

    SOFT_EDGE_WIDTH = 1

    HARD_EDGES_WIDTH = 2

    EDGE_COLOR = 'black'

    MINIMUM_RGB = 40

    MAX_RGB = 240

    MESSAGE_FONT_SIZE = 20

    MESSAGE_FONT = 'Arial'



    def initialize

      @view = Sketchup.active_model.active_view

      @top_message = "#{PLUGIN_NAME} 未找到许可证 - 以预览模式运行。"

      @bottom_message = "点击 扩展程序 / 布尔工具2 / 许可证.. 以激活您的许可证。"

      @top_message_location =  Geom::Point3d.new((@view.vpwidth / 2).to_i, 10, 0)

      @bottom_message_location =  Geom::Point3d.new((@view.vpwidth / 2).to_i, @view.vpheight  - 40, 0)

      @top_message_options = {

          :font => MESSAGE_FONT,

          :size => MESSAGE_FONT_SIZE,

          :bold => true,

          :align => TextAlignCenter

      }

      @bottom_message_options = {

          :font => MESSAGE_FONT,

          :size => MESSAGE_FONT_SIZE,

          :bold => true,

          :align => TextAlignCenter

      }

      @ro = Sketchup.active_model.rendering_options

      reset

    end



    # Sets the bounding box for the invisible bounds

    # Used to help the user orbit around the preview geometry

    def set_bounds(box)

      @bounds = box

    end



    # Draws an invisible bounding box around the preview geometry to help

    # the user orbit

    def draw_invisible_bounds

      p0 = @bounds.corner(0)

      p1 = @bounds.corner(1)

      p2 = @bounds.corner(2)

      p3 = @bounds.corner(3)

      p4 = @bounds.corner(4)

      group = Sketchup.active_model.active_entities.add_group

      @bounds_group = group

      ents = group.entities

      f = ents.add_face(p0, p1, p3, p2)

      f.reverse! if f.normal.samedirection?(Z_AXIS.reverse)

      height = p4.z - p0.z

      f.pushpull(height.to_f)

      group.material = get_preview_cage_material

      edges = ents.grep(Sketchup::Edge)

      edges.each { |edge| edge.hidden = true }

      @ro["DrawSilhouettes"] = false

      @ro["EdgeDisplayMode"] = 0

      @ro["RenderMode"] = 2

    end



    # Returns the invisible cage material

    def get_preview_cage_material

      mats = Sketchup.active_model.materials

      if mats[CAGE_MATERIAL_NAME]

        mats.remove(CAGE_MATERIAL_NAME)

      end

      mat = mats.add(CAGE_MATERIAL_NAME)

      mat.alpha = 0.0

      mat

    end



    # Stores the current rendering settings so we can re-enable them later

    def save_current_rendering_options

      @draw_profiles = @ro["DrawSilhouettes"]

      @draw_edges = @ro["EdgeDisplayMode"]

      @render_mode = @ro["RenderMode"]

    end



    # Restores the preview rendering settings

    def restore_rendering_options

      @ro["DrawSilhouettes"] = @draw_profiles

      @ro["EdgeDisplayMode"] = @draw_edges

      @ro["RenderMode"] = @render_mode

    end



    # Reset this tool to the initial state

    def reset

      @polys = {}

      @hard_edges = {}

      @soft_edges = {}

      @bounds = Geom::BoundingBox.new

      @visible_entities = []

      @bounds_group = nil

      @colors = {}

    end



    def get_entities(inst)

      if inst.is_a?(Sketchup::Group)

        inst.entities

      else

        inst.definition.entities

      end

    end



    # Creates a polygon mesh representation of the instance

    # @param [Group or Component Instance] inst

    # Always returns even if there is an error (to ensure that the actual geometry

    # creation will always get aborted)

    def add_preview_mesh(inst)

    begin

        ents = get_entities(inst)

        @polys[inst] = []

        @hard_edges[inst] = []

        @soft_edges[inst] = []

        @colors[inst] = get_instance_color(@colors.keys.length)

        add_faces_to_mesh(inst, ents, inst.transformation)

      rescue => e

        puts "添加预览网格时出错。"

        Debug.puts e.message

        Debug.puts e.backtrace

      ensure

        return

      end

    end



    def add_faces_to_mesh(inst, ents, trans)

      ents.each { |e|

        if e.is_a?(Sketchup::Group) || e.is_a?(Sketchup::ComponentInstance)

          add_faces_to_mesh(inst, get_entities(e), trans * e.transformation)

        end

        if e.is_a?(Sketchup::Face)

          face_mesh = e.mesh.transform!(trans)

          polys = face_mesh.polygons

          polys.each { |poly|

            @polys[inst] << (face_mesh.point_at(poly[0])) << face_mesh.point_at(poly[1]) << face_mesh.point_at(poly[2])

          }

        elsif e.is_a?(Sketchup::Edge)

          edge = e.vertices.collect { |v| v.position.transform(trans) }

          if e.soft?

            @soft_edges[inst] << edge

          else

            @hard_edges[inst] << edge

          end

        end

      }

    end



    def getExtents

      Sketchup.active_model.bounds

    end



    def draw(view)

      draw_edges(view)

      draw_polys(view)

      draw_trial_message(view)

    end



    def draw_polys(view)

      @polys.each { |inst, polys|

        view.drawing_color = @colors[inst]

        view.draw(GL_TRIANGLES, polys)

      }

    end



    def draw_edges(view)

      draw_soft_edges(view)

      draw_hard_edges(view)

    end



    def draw_trial_message(view)

      view.draw_text(@top_message_location, @top_message, @top_message_options)

      view.draw_text(@bottom_message_location, @bottom_message, @bottom_message_options)

    end



    def draw_soft_edges(view)

      view.drawing_color = EDGE_COLOR

      view.line_width = SOFT_EDGE_WIDTH

      view.line_stipple = "-"

      @soft_edges.each { |inst, edges|

        f_edges = edges.flatten

        view.draw(GL_LINES, f_edges)

      }

    end



    def draw_hard_edges(view)

      view.drawing_color = EDGE_COLOR

      view.line_width = HARD_EDGES_WIDTH

      view.line_stipple = ""

      @hard_edges.each { |inst, edges|

        f_edges = edges.flatten

        view.draw(GL_LINES, f_edges)

      }

    end



    # Returns the color to be used for drawing the various instances

    def get_instance_color(index)

      if index == 0

        # Set the color to light grey if this the first or only instance

        color = Sketchup::Color.new(210, 210, 220, PREVIEW_ALPHA)

      else

        # Set a random color for other instances

        r = rand(MINIMUM_RGB..MAX_RGB)

        g = rand(MINIMUM_RGB..MAX_RGB)

        b = rand(MINIMUM_RGB..MAX_RGB)

        color = Sketchup::Color.new(r, g, b, PREVIEW_ALPHA)

      end

      color

    end



    def resume(view)

      view.invalidate

    end



    def activate

      save_current_rendering_options

      Sketchup.active_model.start_operation("Preview Mode Visibility")

      @visible_entities = Sketchup.active_model.active_entities.find_all { |e| e.visible? }

      @visible_entities.each { |e| e.hidden = true }

      draw_invisible_bounds

      Sketchup.active_model.commit_operation

      @view.invalidate

    end



    # Restore the visibility of the entities when the tool exits

    def deactivate(view)

      Sketchup.active_model.start_operation("Revert Preview Mode Visibility")

      # Restore visibilty but protect against entities that might have been made invalid

      # while this tool was active

      @visible_entities.each { |e| e.visible = true if e.valid? }

      @bounds_group.erase! if @bounds_group.valid?

      restore_rendering_options

      Sketchup.active_model.commit_operation

    end



  end #class previewer



end